<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class serviceHelperDbFabric extends Object {

	private $_resource		= '';
	private $_connected		= false;

	private $_dbHost		= '';
	private $_dbPort		= '';
	private $_dbName		= '';
	private $_dbUser		= '';
	private $_dbPrefix		= '';

	private $_sql			= '';
	private $_debug		= '';
	private $_cursor		= null;
	private $_limit			= 0;
	private $_offset		= 0;
	private $_nameQuote		= null;
	private $_delimiter =null;


	public function __construct() {
		$this->initObj();
		$this->milestone("DB fabric constructed");
		$this->_delimiter=Database::getInstance()->getDelimiter();
	}

	private function replacePrefix($sql,$prefix='#__') {
		$sql = trim($sql);

		$escaped = false;
		$quoteChar = '';

		$n = mb_strlen($sql,DEF_CP);

		$startPos = 0;
		$literal = '';

		while($startPos < $n) {
			$ip = mb_strpos($sql,$prefix,$startPos);
			if ($ip === false) {
				break;
			}

			$j = mb_strpos($sql,"'",$startPos);
			$k = mb_strpos($sql,'"',$startPos);
			if (($k !== false) && (($k < $j) || $j == false)) {
				$quoteChar = '"';
				$j = $k;
			}
			else {
				$quoteChar = "'";
			}

			if ($j === false) {
				$j = $n;
			}

			$literal .= str_replace($prefix,$this->_dbPrefix,substr($sql,$startPos,$j - $startPos));
			$startPos = $j;

			$j = $startPos + 1;
			if ($j >= $n) {
				break;
			}

			while (true) {
				$k = strpos($sql,$quoteChar,$j);
				$escaped = false;
				if ($k === false) {
					break;
				}
				$l = $k - 1;
				while ($l >= 0 && $sql{$l} == '\\') {
					$l--;
					$escaped = !$escaped;
				}
				if ($escaped) {
					$j = $k + 1;
					continue;
				}
				break;
			}

			if ($k == false) {
				// error in the query - no end quote; ignore it
				break;
			}
			$literal .= substr($sql,$startPos,$k - $startPos + 1);
			$startPos = $k + 1;
		}
		if ($startPos < $n) {
			$literal .= substr($sql,$startPos,$n - $startPos);
		}
		return $literal;
	}

	function getNumRows( $cur=null ) {
		return mysql_num_rows( $cur ? $cur : $this->_cursor );
	}
	public function checkValidity() {
		$isValid = true;

		// Check some functions to be defined at PHP interpreter level
		if (function_exists('mysql_connect') == false) $isValid = false;
		if (function_exists('mysql_select_db') == false) $isValid = false;
		if (function_exists('mysql_query') == false) $isValid = false;
		if (function_exists('mysql_close') == false) $isValid = false;

		return $isValid;
	}

	public function isConnected() {
		return $this->_connected;
	}

	public function getPrefix() {
		return $this->_dbPrefix;
	}

	public function getDBName() {
		return $this->_dbName;
	}

	public function reconnect($dbhost,$dbport,$dbname,$dbuser,$dbpass,$dbprefix,$critFail=true) {
		$this->_connected = false;
		if ($this->_resource) mysql_close($this->_resource);

		$this->_dbHost = $dbhost;
		$this->_dbPort = $dbport;
		$this->_dbName = $dbname;
		$this->_dbUser = $dbuser;
		$this->_dbPrefix = $dbprefix;

		if (!($this->_resource = @mysql_connect($dbhost.":".$dbport,$dbuser,$dbpass,true))) {
			if ($critFail)
				$this->fatalError(Text::_("DB connect failed")." params ($dbhost,$dbport,$dbname,$dbuser,$dbpass)");
			else {
				$this->_connected = false;
				return false;
			}
		}

		if (!mysql_select_db($dbname,$this->_resource)) {
			if ($critFail)
				$this->fatalError(Text::_("DB select failed")." ".$dbname);
			else {
				$this->_connected = false;
				return false;
			}
		}

		$this->milestone("DB selected `".$dbname."`");

		@mysql_query("SET NAMES 'UTF8'",$this->_resource);

		$this->_connected = true;
	}

	private function getEscaped($text) {
		if (version_compare(phpversion(), '4.3.0', '<')) {
			$string = mysql_escape_string($text);
		} else 	{
			$string = mysql_real_escape_string($text, $this->_resource);
		}
		return $string;
	}

	/**
	 * Only quote if the name is not using dot-notation
	 * @param unknown_type $s
	 */
	public function nameQuote( $s )
	{
		if (mb_strpos( $s, '.' ) === false)
		{
			$q = $this->_nameQuote;
			if (mb_strlen( $q ,DEF_CP) == 1) {
				return $q . $s . $q;
			} else {
				return $q{0} . $s . $q{1};
			}
		}
		else {
			return $s;
		}
	}

	public function quote($text) {
		return '\''.$this->getEscaped($text).'\'';
	}

	public function setQuery($sql, $offset = 0, $limit = 0, $prefix='#__') {
		$this->_sql = $this->replacePrefix($sql,$prefix);
		$this->_limit = intval($limit);
		$this->_offset = intval($offset);
	}

	public function getQuery() {
		return '<pre>'.htmlspecialchars($this->_sql).'</pre>';
	}

	public function query() {
		if ($this->_limit > 0 && $this->_offset == 0) {
			$this->_sql .= "\nLIMIT ".$this->_limit;
		}
		else if ($this->_limit > 0 || $this->_offset > 0) {
			$this->_sql .= "\nLIMIT ".$this->_offset.", ".$this->_limit;
		}

		if ($this->_debugger) Debugger::getInstance()->sqlQuery($this->_sql);
		$this->_cursor = @mysql_query($this->_sql,$this->_resource);
		if (!$this->_cursor) {
			$this->error(Text::_("DB error")." => query ".$this->_sql." -:- ".mysql_error($this->_resource));
			return false;
		}

		return $this->_cursor;
	}

	public function loadResult() {
		if (!($cur = $this->query())) {
			return null;
		}
		$ret = null;
		if ($row = @mysql_fetch_row($cur)) {
			$ret = $row[0];
		}
		@mysql_free_result($cur);
		return $ret;
	}

	public function loadAssocList($key='') {
		if (!($cur = $this->query())) {
			return null;
		}
		$array = array();
		while ($row = @mysql_fetch_assoc($cur)) {
			if ($key) {
				$array[$row[$key]] = $row;
			} else {
				$array []= $row;
			}
		}
		@mysql_free_result($cur);
		return $array;
	}

	/**
	 * Load an array of single field results into an array
	 */
	function loadResultArray($numinarray = 0) {
		if (!($cur = $this->query())) {
			return null;
		}
		$array = array();
		while ($row = mysql_fetch_row( $cur )) {
			$array[] = $row[$numinarray];
		}
		mysql_free_result( $cur );
		return $array;
	}

	public function loadObject(&$object) {
		if ($object != null) {
			if (!($cur = $this->query())) {
				return false;
			}
			if ($array = mysql_fetch_assoc($cur)) {
				@mysql_free_result($cur);
				foreach ($array as $key=>$value) {
					$object->$key = $value;
				}
				return true;
			} else {
				return false;
			}
		} else {
			if ($cur = $this->query()) {
				if ($object = @mysql_fetch_object($cur)) {
					@mysql_free_result($cur);
					return true;
				} else {
					$object = null;
					return false;
				}
			} else {
				return false;
			}
		}
	}

	public function loadObjectList($key='') {
		if (!($cur = $this->query())) {
			return null;
		}
		$array = array();
		while ($row = @mysql_fetch_object($cur)) {
			if ($key) {
				$array[$row->$key] = $row;
			} else {
				$array[] = $row;
			}
		}
		@mysql_free_result($cur);
		return $array;
	}

	public function insertid() {
		return mysql_insert_id( $this->_resource );
	}

	public function now() {
		$query = "SELECT NOW()";
		$this->setQuery($query);
		return $this->loadResult();
	}

	public function truncate($table=false)
	{
		if($table)
		{
			$this->setQuery("truncate table #__$table");
			return $this->query();
		}
		else return false;
	}
	/**
	 * маска полей по таблице для пробежки при инсерте и апдейте
	 * @param $table
	 * возвращает массив объектов по ключу имени поля
	 */
	public function getListFieldfromTable($table)
	{
		$this->setQuery("describe #__".$table);
		$res_field=$this->LoadObjectList('Field');
		return $res_field;
	}

	public function sql_table_exist($table = "")
	{
		$tbl_exist=false;
		//if( $this->db_connect_id )
		//{

		$row = mysql_query("SHOW TABLES");
		while ($line = mysql_fetch_array($row))
		{
			if($line[0]==$table) $tbl_exist=true;
		}
		mysql_free_result($row);
		//return $tbl_exist;
		//}
		return $tbl_exist;
	}
	/**
	 * Diagnostic function
	 */
	public function explain() {
		$temp = $this->_sql;
		$this->_sql = "EXPLAIN $this->_sql";
		$this->query();

		if (!($cur = $this->query())) {
			return null;
		}
		$first = true;

		$buf = "<table cellspacing=\"1\" cellpadding=\"2\" border=\"0\" bgcolor=\"#000000\" align=\"center\">";
		$buf .= $this->getQuery();
		while ($row = mysql_fetch_assoc( $cur )) {
			if ($first) {
				$buf .= "<tr>";
				foreach ($row as $k=>$v) {
					$buf .= "<th bgcolor=\"#ffffff\">$k</th>";
				}
				$buf .= "</tr>";
				$first = false;
			}
			$buf .= "<tr>";
			foreach ($row as $k=>$v) {
				$buf .= "<td bgcolor=\"#ffffff\">$v</td>";
			}
			$buf .= "</tr>";
		}
		$buf .= "</table><br />&nbsp;";
		mysql_free_result( $cur );

		$this->_sql = $temp;

		return "<div style=\"background-color:#FFFFCC\" align=\"left\">$buf</div>";
	}
	public function query_batch( $abort_on_error=true, $p_transaction_safe = false,$delimeter='') {
		if (!$delimiter) $delimiter=$this->_delimiter;
		$this->_errorNum = 0;
		$this->_errorMsg = '';
		if ($p_transaction_safe) {
			$si = mysql_get_server_info( $this->_resource );
			preg_match_all( "/(\d+)\.(\d+)\.(\d+)/i", $si, $m );
			if ($m[1] >= 4) {
				$this->_sql = 'START TRANSACTION'.$delimeter. $this->_sql .$delimeter.'COMMIT'.$delimeter;
			} else if ($m[2] >= 23 && $m[3] >= 19) {
				$this->_sql = 'BEGIN WORK'.$delimeter . $this->_sql .$delimeter.' COMMIT'.$delimeter;
			} else if ($m[2] >= 23 && $m[3] >= 17) {
				$this->_sql = 'BEGIN'.$delimeter. $this->_sql .$delimeter. 'COMMIT'.$delimeter;
			}
		}
		$query_split = preg_split ("/".$delimeter."/", $this->_sql);
		$error = 0;
		if ($this->_debugger) Debugger::getInstance()->sqlQuery($this->_sql);
		foreach ($query_split as $command_line) {
			$command_line = trim( $command_line );
			if ($command_line != '') {
				if ($this->_debug) {
					$this->_ticker++;
					$this->_log[] = $this->_db_name.":".$command_line;
				}

				$this->_cursor = mysql_query( $command_line, $this->_resource );
				if (!$this->_cursor) {
					$error = 1;
					//echo mysql_error( $this->_resource )." SQL=$command_line <br />";
					//exit;
					$this->_errorNum .= mysql_errno( $this->_resource ) . ' ';
					$this->_errorMsg .= mysql_error( $this->_resource )." SQL=$command_line <br />";
					if ($this->_debug) {
						//echo "<pre>" . $this->_sql . "</pre>\n";
						trigger_error( mysql_error( $this->_resource ), E_USER_NOTICE );
						if (function_exists( 'debug_backtrace' )) {
							foreach( debug_backtrace() as $back) {
								if (@$back['file']) {
									echo '<br />'.$back['file'].':'.$back['line'];
								}
							}
						}
					}
					if ($abort_on_error) {
						return $this->_cursor;
					}
				}
			}
		}
		return $error ? false : true;
	}

	public function getLastError()
	{
		return mysql_error($this->_resource);
	}

}

?>