//ALTADOC_COPYRIGHT_TEMPLATE

// Элемент управления "Информация"
function InformationControl() {
	var geoResult, clickPlace, listener, map;

	// Вызывается при добавлении элемента управления на карту
	this.onAddToMap = function(parentMap) {
		map = parentMap;

		map.addCursor(YMaps.Cursor.HELP);

		// Создание обработчика кликов по карте
		listener = YMaps.Events.observe(map, map.Events.Click, function(map,
				mEvent) {

			// Координаты клика мышью
				var clickPoint = mEvent.getGeoPoint();

				// Удаляем предыдущие результаты (если они были добавлены на карту)
				if (geoResult) {
					map.removeOverlay(geoResult);
					result = null;
				}

				if (clickPlace) {
					map.removeOverlay(clickPlace);
					clickPlace = null;
				}

				// Отмечаем точку по которой щелкнул пользователь
				clickPlace = new YMaps.Placemark(clickPoint, {
					style : anchorStyle
				});
				clickPlace.description = clickPoint.toString();
				map.addOverlay(clickPlace);

				// Запуск процесса геокодирования
				this.geocode(clickPoint);
			}, this);
	}

	// Геокодирует точку
	this.geocode = function(clickPoint) {
		// Выключаем обработчиков событий, чтобы к геокодеру ушло не более одного запроса
		// (по окончанию геокодированияю включаем обработчик вновь)
		listener.disable();

		// Запуск процесса геокодирования
		var geocoder = new YMaps.Geocoder(clickPoint);

		// Обработчик успешного завершения геокодирования
		YMaps.Events
				.observe(
						geocoder,
						geocoder.Events.Load,
						function(geocoder) {
							// Получение результата поиска
							geoResult = this.getResult(geocoder);

							if (geoResult) {
								geoResult.setStyle(markStyle);

								// Задаем содержимое балуна
								var sep = ', ', names = (geoResult.text || '')
										.split(sep), index = geoResult.kind === 'house' ? -2
										: -1;
								geoResult.setBalloonContent("<b>"
										+ names.slice(index).join(sep)
										+ "</b><div>"
										+ names.slice(0, index).join(sep)
										+ " ("+ geoResult._point+")</div>");

								// Открываем балун
								map.addOverlay(geoResult);
								geoResult.openBalloon();
							} else {
								alert("Ничего не найдено!");
							}

							// Включаем обработчик кликов по карте
							listener.enable();
						}, this);

		// Обработчик неудачного геокодирования
		YMaps.Events.observe(geocoder, geocoder.Events.Fault, function(
				geocoder, err) {
			alert("Произошла ошибка при геокодировании: " + err);

			// Включаем обработчик кликов по карте
				listener.enable();
			});
	}

	// Возвращает результат различной точности в зависимости от масштаба
	this.getResult = function(geocoder) {
		// Точность: город, страна
		function isOther(result) {
			return result.precision == "other";
		}

		// Точность: улица
		function isStreet(result) {
			return result.precision == "street";
		}

		// Точность: дом
		function isHouse(result) {
			return !isOther(result) && !isStreet(result);
		}
		;

		// Выбор точности поиска
		var filter = isHouse;
		if (map.getZoom() < 10) {
			filter = isOther;
		} else if (map.getZoom() < 15) {
			filter = isStreet;
		}

		// Возвращает первый найденный результат с нужной точностью
		return geocoder.filter(filter)[0];
	}

	// Вызывается при удалении элемента управления с карты
	this.onRemoveFromMap = function() {
		map.removeCursor(YMaps.Cursor.HELP);

		// Удаляем метки с карты, если они были добавлены
		if (geoResult) {
			map.removeOverlay(geoResult);
		}

		if (clickPlace) {
			map.removeOverlay(clickPlace);
		}

		// Удаляем обработчик кликов по карте
		listener.cleanup();

		map = geoResult = clickPlace = listener = null;
	}

	// Создадим стили для значков
	var anchorStyle = new YMaps.Style();
	anchorStyle.iconStyle = new YMaps.IconStyle();
	anchorStyle.iconStyle.href = "http://maps.yandex.ru/css/b-location-balloon/b-location-balloon.anchor.png";
	anchorStyle.iconStyle.size = new YMaps.Point(18, 18);
	anchorStyle.iconStyle.offset = new YMaps.Point(-9, -18);

	var markStyle = new YMaps.Style();
	markStyle.iconStyle = new YMaps.IconStyle();
	markStyle.iconStyle.href = "http://maps.yandex.ru/css/b-location-balloon/b-location-balloon.mark.png";
	markStyle.iconStyle.size = new YMaps.Point(21, 19);
	markStyle.iconStyle.offset = new YMaps.Point(-1, -18);
}
function showPointOnMap(value,psid,html,bb) {
	if (bb===undefined) var bb='default#houseIcon';

    // Удаление предыдущего результата поиска
    map.removeOverlay(geoResult);
    // Запуск процесса геокодирования
    var geocoder = new YMaps.Geocoder(value, {results: 1, boundedBy: map.getBounds()});
    var geocoderRes="";
    // Создание обработчика для успешного завершения геокодирования
    YMaps.Events.observe(geocoder, geocoder.Events.Load, function () {
        // Если объект был найден, то добавляем его на карту
        // и центрируем карту по области обзора найденного объекта
        if (this.length()) {
            geoResult = this.get(0);
			var sep = ', ', names = (geoResult.text || '').split(sep), index = geoResult.kind === 'house' ? -2 : -1;
			geocoderRes=geoResult._point.toString();
			geoResult.setBalloonContent("<b>"+ names.slice(index).join(sep)	+ "</b>"
								+ "<div style=\"height:60px; width:250px;\">"
								+ names.slice(0, index).join(sep)
//								+ "<input type=\"hidden\" value=\"" + geocoderRes + "\"  id=\"geocoderRes_" + psid +"\" />"
								+ "</div>"
								+ html);
			$("#geocoderRes_" + psid).val(geocoderRes);
			geoResult.setStyle(bb);
            map.addOverlay(geoResult);
            map.setBounds(geoResult.getBounds(),16);
        }else {
            alert("Ничего не найдено");
        }
    });

    // Процесс геокодирования завершен неудачно
    YMaps.Events.observe(geocoder, geocoder.Events.Fault, function (geocoder, error) {
        alert("Произошла ошибка: " + error);
    });
}	
