<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Basket extends Object {
	public static $cookie_name = "altadoc_basket";
	private $taxes = null;
	private $taxesInOrder = array();
	public $goods = null; // id и количество товаров из куки
	public $items = null; // рассчитанная таблица товаров и услуг
	public $deliverySum = 0;
	public $paymentSum = 0;
	public $taxesSum = 0;
	public $discountSum = 0;
	public $points = 0;
	public $points_enabled=false;
	public $summa = 0;
	public $total = 0;
	public $quantity = 0;
	public $weight = 0;
	public $order_complete = 0;
	public $order_id = 0;
	public $order_message = 0;
	public $order_vendor = 0;

	//---------- Singleton implementation ------------
	private static $_instance = null;

	public static function createInstance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
		}
	}

	public static function getInstance() {
		self::createInstance();
		return self::$_instance;
	}
	//------------------------------------------------

	private function __construct() {
		$this->initObj();
		$this->parseCookie();
		$this->points_enabled=siteConfig::$use_points_system;
		if (!isset($_SESSION["basket_vendor"])) $_SESSION["basket_vendor"]=0;
	}

	public function cleanCookie() {
		if($this->order_complete) {
			if (catalogConfig::$multy_vendor) {
				$vendor=$_SESSION["basket_vendor"];
				foreach($this->items as $item){
					if ($item["g_vendor"]==$vendor)	$this->goods[$item["g_id"]]=0;
				}
				$this->updateCookie();
			} else {
				setcookie(self::$cookie_name,'',time()-60*60*24*30,'/');
			}
			unset($_SESSION["altadoc_order"]);
			unset($_SESSION["basket_vendor"]);
		}
	}
	private function parseCookie() {
		$this->goods=null;
		if (isset($_COOKIE[self::$cookie_name])) {
			$cook=$_COOKIE[self::$cookie_name];
			$goods_arr=explode("#", $cook);
			if (is_array($goods_arr) && count($goods_arr)>0) {
				foreach($goods_arr as $gstring) {
					$tmp_goods=explode("|", $gstring);
					if ((is_array($tmp_goods))&&(count($tmp_goods)==2)) {
						$this->goods[$tmp_goods[0]]=$tmp_goods[1];
					}
				}
			}
		}
	}
	private function updateCookie(){
		$result="";
		$new_goods=array();		
		if (($this->goods)&&(count($this->goods)>0))	{
			foreach ($this->goods as $key=>$val) {
				if ($val>0) {
					//todo urlencode наверное можно убрать
					$result.=urlencode($key)."|".$val.'#';
					$new_goods[$key]=$val;
				}
			}
			if (mb_strlen($result)>0) $result=mb_substr($result,0,mb_strlen($result)-1);
			//      $result=json_encode($new_goods);
			setcookie(self::$cookie_name,$result,time()+60*60*24*30,'/');
			$this->goods=$new_goods;
		} else {
			setcookie(self::$cookie_name, "", time() - 3600);
		}
	}
	public function updateBasketPosition ($psid, $quantity) {
		if ($this->goods && count($this->goods) && array_key_exists($psid, $this->goods)) {
			$this->goods[$psid]=$quantity;
			$this->updateCookie();
		}
	}
	public function deleteBasketPosition ($psid) {
		$this->updateBasketPosition($psid, 0);
	}
	private function showEmptyBasket() {
		return "<div id=\"mybasket\" class=\"empty_basket\">".Text::_('Basket is empty')."</div>";
	}
	// мелкая корзинка для виджета
	public function showMini() {
		$summa=0; $this->points=0; $_html="";
		if (catalogConfig::$multy_vendor) {
			if ($_SESSION["basket_vendor"]==0) {
				$vendor_ids=$this->getGoodsVendorsFromDB();
			} else {
				$vendor_ids=array($_SESSION["basket_vendor"]);
			}
		} else {
			$vendor_ids=array(catalogConfig::$default_vendor);
		}
		$this->updateCookie();
		if (($this->goods)&&(count($this->goods)>0))	{
			foreach($vendor_ids as $vendor_id) {
				$garr=$this->getGoodsFromDB($vendor_id);
				if (count($garr)>0) {
					foreach($garr as $g) {
						//  				$current_quantity=$this->goods[$g->g_id];
						if ($g->current_quantity) {
							$current_price=$this->getPrice($g);
							//  					$current_quantity=round($this->goods[$g->g_id],catalogConfig::$quantity_digits);
							//  					if ($g->g_isweight)	$current_price=number_format($current_price*$g->g_weight,2);
							$current_sum=$current_price*$g->current_quantity;
							$summa=$summa+$current_sum;
							$_html .= "<p class=\"goods_row\">".$g->g_name." x ".number_format($g->current_quantity,catalogConfig::$quantity_digits)." ".Measure::getInstance()->getShortName($g->g_sell_measure)."</p>";
						}
					}
					$_html .= "<p class=\"subtotal\">".Text::_('Sum')." : ".number_format($summa,2)."&nbsp;".Currency::getShortName(DEFAULT_CURRENCY)."</p> \n";
					$_html .= "<p class=\"buttons\"><a rel=\"nofollow\" class=\"linkButton\" href=\"".Router::_("index.php?module=catalog&amp;view=orders&amp;layout=basket")."\">".Text::_('Modify')."/".Text::_('Make order')."</a></p> \n";
				} else { 
					$_html=$this->showEmptyBasket();	
				}
			}
		} else { $_html=$this->showEmptyBasket();
		}
		return $_html;
	}


	public function modifyBasket($ajax=0, $read_only=true,$no_buttons=true) {
		$basket_div="mybasket";
		if ($ajax) $basket_div="mybasket_ajax";
		$this->updateCookie();
		$colspan=6;
		if ($this->points_enabled) $colspan=$colspan+1;
		if (($this->goods)&&(count($this->goods)>0))	{
			if (catalogConfig::$multy_vendor) {
				if ($ajax || ($_SESSION["basket_vendor"]==0)) {
					$vendor_ids=$this->getGoodsVendorsFromDB();
					$read_only=false;
					$no_buttons=false;
				} else {
					$vendor_ids=array($_SESSION["basket_vendor"]);
				}
			} else {
				if ($ajax || $_SESSION["basket_vendor"]==0) {
					$read_only=false;
					$no_buttons=false;
				}
				$vendor_ids=array(catalogConfig::$default_vendor);
			}
			//$vendors_ids=$this->getGoodsVendorsFromDB();
			//$vendors_ids=$this->getGoodsVendorsFromDB();
			$_html = "<div id=\"".$basket_div."\"> \n\n";
			if (count($vendor_ids)>1) {
				$_html .= "<h4 class=\"basket_vendor_caution\">";
				$_html .= Text::_("You selected goods from several vendors");
				$_html .= ".<br />";
				$_html .= Text::_("They are splitting to several orders");
				$_html .= ".</h4>";
			}
			$vendors_found=0; $goods_found=array();
			foreach($vendor_ids as $vendor_id) {
				$summa=0;
				$this->points=0;
				$garr=$this->getGoodsFromDB($vendor_id);
				if (count($garr)>0) {
					$vendors_found++;
					$_html .= "<table class=\"goods\"> \n\n";
					$_html .= "<thead> \n";
					$vendor=Vendor::getInstance()->getVendor($vendor_id);
					$_html .= "<tr> \n";
					$_html .= "<td class=\"basket_vendor\" colspan=\"".$colspan."\"> \n";
					$_html .= Text::_("Vendor")." : ".$vendor->v_name;
					$_html .= "</td></tr>";
					$_html .= "<tr> \n";
					$_html .= "<th>".Text::_('Articul')."</th> \n";
					$_html .= "<th>".Text::_('Goods title')."</th>";
					if ($this->points_enabled) {
						$_html .= "<th>".Text::_('Points')."</th>";
					}
					$_html .= "<th>".Text::_('Price')."</th>";
					$_html .= "<th>".Text::_('Quantity')."</th>";
					$_html .= "<th>".Text::_('Measure')."</th>";
					$_html .= "<th>".Text::_('Sum')."</th>";
					$_html .= "</tr>";
					$_html .= "</thead><tbody>";
					foreach($garr as $g) {
						//		$current_quantity=$this->goods[$g->g_id];
						if ($g->current_quantity) {
							$goods_found[$g->g_id]=$g->current_quantity;
							$_html .= "<tr>";
							$_html .= "<td>".$g->g_sku."</td>";
							$_html .= "<td>".$g->g_name."</td>";
							$current_price=$this->getPrice($g);
							//	            if ($g->g_isweight){ $current_price=round($current_price*$g->g_weight,2);  }
							//            $current_quantity=round($this->goods[$g->g_id],catalogConfig::$quantity_digits);
							$current_sum=$current_price*$g->current_quantity;
							if ($this->points_enabled) {
								$current_points=$g->g_points*$g->current_quantity;
								$this->points=$this->points+$current_points;
								$_html .= "<td class=\"cur_points\" >".$current_points."</td>";
							}
							$_html .= "<td class=\"cur_price\">".number_format($current_price,2)."</td>";
							$summa=$summa+$current_sum;
							$_html .= "<td class=\"cur_quantity\">";
							if ($read_only) {
								$_html.=number_format($g->current_quantity,catalogConfig::$quantity_digits);
							} else {
								$_html .= "<input size=\"10\" id=\"gq_".$g->g_id."\" name=\"gq_".$g->g_id."\" type=\"text\" value=\"".number_format($g->current_quantity,catalogConfig::$quantity_digits)."\" />";
								$_html .= "<a class=\"b_link_save\" onclick=\"javascript:updateBasketPosition('".$g->g_id."','gq_".$g->g_id."','".$basket_div."');\" title=\"".Text::_('Update')."\"><img src=\"/images/blank.gif\" alt=\"U\" width=\"16\" height=\"16\" /></a>";
							}
							$_html .= "</td>";
							$_html .= "<td class=\"cur_measure\">".Measure::getInstance()->getShortName($g->g_sell_measure)."</td>";
							$_html .= "<td class=\"cur_sum\" id=\"sum_".$g->g_id."\">".number_format($current_sum,2);
							if (!$read_only) {
								$_html .= "&nbsp;<a  class=\"b_link_del\" onclick=\"javascript:deleteBasketPosition('".$g->g_id."','".$basket_div."');\" title=\"".Text::_('Delete')."\"><img src=\"/images/blank.gif\" alt=\"D\" width=\"16\" height=\"16\" /></a>";
							}
							$_html .= "</td>";
							$_html .= "</tr> \n";
						}
					}
					$_html .= "<tr> \n";
					if ($this->points_enabled) {
						$_html .= "<td colspan=\"3\" class=\"subtotal\">".Text::_('Total')." : ".$this->points."</td> \n";
					}
					else $_html .= "<td class=\"subtotal\" colspan=\"2\">&nbsp;</td> \n";
					$_html .= "<td class=\"subtotal\" colspan=\"4\">".Text::_('Sum')." : ".number_format($summa,2)."&nbsp;".Currency::getShortName(DEFAULT_CURRENCY)."</td> \n";
					$_html .= "</tr>";
					if (!$no_buttons) {
						$_html .= "<tr> \n";
						$_html .= "<td class=\"buttons\" colspan=\"".$colspan."\"> \n";
						$_html .= "<a id=\"submit\" class=\"linkButton\" href=\"".Router::_("index.php?module=catalog&amp;view=orders&amp;layout=basket&amp;vendor=".$vendor_id)."\">".Text::_('Proceed order')."</a> \n";
						$_html .= "</td>";
						$_html .= "</tr>";
					}
					$_html .= "</tbody>";
					$_html .= "</table>";
				}
			}
			$_html .= "</div>";
			$this->goods = $goods_found;
			$this->updateCookie();
			if (!$vendors_found) {$_html=$this->showEmptyBasket();}
		} else { 
			$_html=$this->showEmptyBasket();
		}
		return $_html;
	}
	public function renderFullOrder($pt, $dt) {
		$_html=""; $counter=0;
		if (($this->goods)&&(count($this->goods)>0))	{
			$garr=$this->calculateOrder($pt, $dt);
			if (count($garr)>0) {
				$_html .= "<div id=\"myorder\" class=\"full_basket\"><table class=\"goods\"> \n\n";
				$_html .= "<thead> \n";
				$_html .= "<tr> \n";
				$_html .= "<th>".Text::_('NPP')."</th> \n";
				$_html .= "<th>".Text::_('Articul')."</th> \n";
				$_html .= "<th>".Text::_('Title')."</th>";
				$_html .= "<th class=\"cur_price\">".Text::_('Price')."</th>";
				if ($this->points_enabled) {
					$_html .= "<th>".Text::_('Points')."</th>";
				}
				$_html .= "<th class=\"cur_quantity\">".Text::_('Quantity')."</th>";
				$_html .= "<th>".Text::_('Measure')."</th>";
				$_html .= "<th class=\"cur_sum\">".Text::_('Sum')."</th>";
				$_html .= "</tr>";
				$_html .= "</thead><tbody>";
				foreach($garr as $g) {
					if ($g["g_quantity"]) {
						$counter++;
						$_html .= "<tr>";
						$_html .= "<td>".$counter."</td>";
						$_html .= "<td>".$g["g_sku"]."</td>";
						$_html .= "<td>".$g["g_name"]."</td>";
						$_html .= "<td class=\"cur_price\">".number_format($g["g_price"],2)."</td>";
						if ($this->points_enabled) $_html .= "<td class=\"cur_points\" >".$g["g_points"]."</td>";
						$_html .= "<td class=\"cur_quantity\">".number_format($g["g_quantity"], catalogConfig::$quantity_digits)."</td>";
						$_html .= "<td class=\"cur_measure\">".Measure::getInstance()->getShortName($g["g_sell_measure"])."</td>";
						$_html .= "<td class=\"cur_sum\">".number_format($g["g_sum"],2)."</td>";
						$_html .= "</tr> \n";
					}
				}
				$colspan=5;
				if ($this->points_enabled) $colspan=$colspan+1;
				$_html .= "<tr>";
				if ($this->points_enabled) {
					$_html .= "<td colspan=\"".($colspan-1)."\" class=\"subtotal\">".Text::_('Total')." : ".$this->points."</td> \n";
				}
				else $_html .= "<td class=\"subtotal\" colspan=\"".$colspan."\">".Text::_('Sum by order')." :</td> \n";
				$_html .= "<td class=\"subtotal\" colspan=\"2\">".number_format($this->summa,2)."</td></tr> \n";

				if (($this->deliverySum)&&(($this->deliverySum)>0)) {
					$_html .= "<tr><td class=\"deliveries\" colspan=\"".$colspan."\">".Text::_("Delivery sum")."</td>";
					$_html .= "<td class=\"deliveries\" colspan=\"2\">".number_format($this->deliverySum,2)."</td></tr> \n";
				}

				if (($this->paymentSum)&&($this->paymentSum>0)) {
					$_html .= "<tr><td class=\"payments\" colspan=\"".$colspan."\">".Text::_("Payment commission")."</td>";
					$_html .= "<td class=\"payments\" colspan=\"2\">".number_format($this->paymentSum,2)."</td></tr> \n";
				}

				if ($this->discountSum) {
					if ($this->discountSum>0) {
						$_html .= "<tr><td class=\"discounts\" colspan=\"".$colspan."\">".Text::_("Discount")." : "."</td>";
						$_html .= "<td class=\"discounts\" colspan=\"2\">".number_format($this->discountSum,2)."</td></tr> \n";
					} else {
						$_html .= "<tr><td class=\"discounts\" colspan=\"".$colspan."\">".Text::_("Fee")." : "."</td>";
						$_html .= "<td class=\"discounts\" colspan=\"2\">".number_format(abs($this->discountSum),2)."</td></tr> \n";
					}
				}
				if($this->total!=$this->summa){
					$_html .= "<tr><td class=\"subtotal\" colspan=\"".$colspan."\">".Text::_("Sum")." : "."</td>";
					$_html .= "<td class=\"subtotal\" colspan=\"2\">".number_format($this->total,2)."</td></tr> \n";
				}
				if (count($this->taxesInOrder)&&$this->taxesSum) {
					$_html .= "<tr><td class=\"taxes\" colspan=\"".$colspan."\">".Text::_("Taxes in order")." : "."</td>";
					$_html .= "<td class=\"taxes\" colspan=\"2\">".number_format($this->taxesSum,2)."</td></tr> \n";
					foreach($this->taxesInOrder as $kt=>$vt){
						$_html .= "<tr><td class=\"taxes\" colspan=\"".$colspan."\">".$this->taxes[$kt]->t_name." : "."</td>";
						$_html .= "<td class=\"taxes\" colspan=\"2\">".number_format($vt,2)."</td></tr> \n";
					}
				}
				$_html .= "<tr><td class=\"total\" colspan=\"".$colspan."\">".Text::_("Total")." : "."</td>";
				$_html .= "<td class=\"total\" colspan=\"2\">".number_format($this->total,2)."&nbsp;".Currency::getShortName(DEFAULT_CURRENCY)."</td></tr> \n";
				$_html .= "</tbody>";
				$_html .= "</table></div>";
			} else { $_html =$this->showEmptyBasket();
			}
		} else { $_html = $this->showEmptyBasket();
		}
		return $_html;
	}

	private function getGoodsFromDB($vendor_id=0) {
		$db = Database::getInstance();
		$result = array(); // $art_arr=null;
		if ($vendor_id && is_array($this->goods)&&(count($this->goods)>0))	{
			$spisok="'".implode("','", array_keys($this->goods))."'";
			$sql = "SELECT * FROM #__goods WHERE g_id IN (".$spisok.")";
			$sql.=" AND g_vendor=".(int)$vendor_id;
			//if (catalogConfig::$multy_vendor && $vendor_id)
			$sql.=" ORDER BY g_vendor";
			$db->setQuery($sql);
			$result = $db->loadObjectList();
			if (count($result)) {
				foreach($result as $g){
					$g->current_quantity=round($this->goods[$g->g_id],catalogConfig::$quantity_digits);
					switch($g->g_selltype){
						case 1:
							$g->g_sell_measure=$g->g_pack_measure;
							break;
						case 2:
							$g->g_sell_measure=$g->g_wmeasure;
							break;
						case 3:
							$g->g_sell_measure=$g->g_vmeasure;
							break;
						case 4:
							//	 					break;
						case 5:
							//						break;
						case 0:
						default:
							$g->g_sell_measure=$g->g_measure;
							break;
					}
				}
			}
		}
		return $result;
	}
	private function getGoodsVendorsFromDB() {
		$db = Database::getInstance();
		$result = array();
		if (is_array($this->goods)&&(count($this->goods)>0))	{
			$spisok="'".implode("','", array_keys($this->goods))."'";
			$sql = "SELECT DISTINCT(g_vendor) FROM #__goods WHERE g_id IN (".$spisok.") and g_vendor>0 ORDER BY g_vendor";
			$db->setQuery($sql);
			$result = array_keys($db->loadObjectList("g_vendor"));
		}
		return $result;
	}
	public function calculateOrder($pt, $dt){
		$_goods=$this->calculateGoods();
		// @FIXME Проверить порядок следования
		$this->deliverySum=$this->calculateDeliverySum($dt,false,$_goods);
		$this->total=$this->summa+$this->deliverySum;

		$this->paymentSum=$this->calculatePaymentSum($pt,false,$_goods);
		$this->total=$this->total+$this->paymentSum;

		$this->discountSum=$this->calculateDiscountSum();
		$this->total=$this->total-$this->discountSum;

		return $_goods;
	}

	public function getGoods(){
		if ($this->items==null) return $this->calculateGoods();
		else return $this->items;
	}

	private function calculateGoods(){
		// попробуем вариант чтобы не пересчитывать с нуля
		$this->items=null;
		$this->summa=0; $this->taxes=0; $this->total=0; $this->points=0;
		$this->quantity = 0;  $this->weight = 0;
		$this->prepareTaxes();
		$vendor_id=$_SESSION["basket_vendor"];
		$garr=$this->getGoodsFromDB($vendor_id);
		if (count($garr)>0) {
			foreach($garr as $g) {
				$cur_price = $this->getPrice($g);
				//        if ($g->g_isweight) $cur_price=round($cur_price*$g->g_weight,2);
				$cur_summa = round($cur_price*$g->current_quantity,2);
				if ($this->points_enabled) {
					$cur_points = round($g->g_points*$g->current_quantity,2);
				} else $cur_points=0;
				$curr_weight=$g->g_weight*$g->current_quantity;
				///        @TODO Лажа тут (только в чем она никто уже не помнит)
				$this->items[]=array('g_id'=>$g->g_id,
						'g_sku'=>$g->g_sku,
						'g_name'=>$g->g_name,
						'g_price'=>$cur_price,
						'g_points'=>$cur_points,
						'g_measure'=>$g->g_measure,
						'g_sell_measure'=>$g->g_sell_measure,
						'g_quantity'=>$g->current_quantity,
						'g_tax'=>$this->calculateTaxSum($g->g_tax,$cur_summa),
						'g_tax_name'=>$this->getTaxName($g->g_tax),
						'g_weight'=>$curr_weight,
						'g_wmeasure'=>$g->g_wmeasure,
						'g_width'=>$g->g_width,
						'g_length'=>$g->g_length,
						'g_height'=>$g->g_height,
						'g_vendor'=>$g->g_vendor,
						'g_sum'=>$cur_summa
				);
				$this->quantity = $this->quantity + Measure::getInstance()->convert($g->current_quantity,$g->g_measure,catalogConfig::$default_measure);
				$this->weight = $this->weight + Measure::getInstance()->convert($curr_weight,$g->g_wmeasure,catalogConfig::$default_wmeasure);
				$this->summa=$this->summa+$cur_summa;
				$this->points=$this->points+$cur_points;
			}
		}

		return $this->items;
	}
	private function calculateDeliverySum($dt,$order,$items) {
		$dt->assignOrder($order,$items);
		$price=$dt->calculate();
		$val=Currency::getInstance()->convert($price,$dt->getCurrency());
		return $val;
	}
	private function calculatePaymentSum($pt,$order,$items) {
		$pt->assignOrder($order,$items);
		$price=$pt->calculate();
		$val=Currency::getInstance()->convert($price,$pt->getCurrency());
		return $val;
	}
	private function getTaxName($tax_id){
		if(!$this->taxes) return "";
		if(!array_key_exists($tax_id, $this->taxes)) return "";
		return $this->taxes[$tax_id]->t_name;

	}
	private function calculateTaxSum($tax_id,$current_sum){
		if(!$this->taxes||!$current_sum) return 0;
		if(!array_key_exists($tax_id, $this->taxes)) return 0;
		$sumtax=round($current_sum/(100+$this->taxes[$tax_id]->t_value)*$this->taxes[$tax_id]->t_value,2);
		if(isset($this->taxesInOrder[$tax_id])) $this->taxesInOrder[$tax_id]+=$sumtax;
		else $this->taxesInOrder[$tax_id]=$sumtax;
		$this->taxesSum+=$sumtax;
		return $sumtax;
	}
	private function calculateDiscountSum(){
		// здесь считается только персональная скидка юзера
		return round($this->total*(User::getInstance()->u_discount/100),2);
	}
	private function prepareTaxes(){
		$db = Database::getInstance();
		$sql = "SELECT * FROM #__taxes WHERE t_enabled=1 AND t_deleted=0";
		$db->setQuery($sql);
		$this->taxes=$result=$db->loadObjectList("t_id");
	}
	private function getPrice($g) {
		$tp = User::getInstance()->u_pricetype;
		$price_name="g_price_".$tp;
		if ($g->g_type==5 && catalogConfig::$complectPriceAsGoodsSum){
			$price_val=$this->getPriceAsSum($g->g_id, $tp);
		} else 	$price_val=$g->$price_name;
		$_val=Currency::getInstance()->convert($price_val, $g->g_currency);
		return $_val;
	}
	private function getPriceAsSum($gid, $tp){
		$db = Database::getInstance();
		$g_price=0;
		$sql="SELECT g.g_id, g.g_price_".$tp." AS g_price, s.s_quantity";
		$sql.=" FROM #__goods_sets AS s, #__goods AS g";
		$sql.=" WHERE s.s_id=g.g_id AND s.g_id=".$gid;
		$db->setQuery($sql);
		$gsets = $db->loadObjectList();
		if(count($gsets)){
			foreach($gsets as $gset) {
				$g_price=$g_price+$gset->g_price*$gset->s_quantity;
			}
		}
		return $g_price;
	}
	
}
?>
