<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Plugin extends Object {
	private static $_loadedPlugins		= array();

	private static $_installedPlugins	= array();
	private static $_corruptedPlugins	= array();

	protected $_events=array();
	protected $_loadedConfig		= array();

	public static function initialize() {
		Debugger::getInstance()->milestone("Loading plugins");
		$db = Database::getInstance();
		$db->setQuery("SELECT * FROM #__plugins WHERE p_deleted=0 ORDER by p_ordering");
		$_installedPlugins = $db->loadObjectList();
		if (count($_installedPlugins)) {
			foreach ($_installedPlugins as $plg) {
				$name=$plg->p_path.".".$plg->p_name;
				$pluginScriptPath = PATH_PLUGINS.$plg->p_path.DS.$plg->p_name.'.php';
				if (file_exists($pluginScriptPath)) {
					require_once $pluginScriptPath;
					$pluginClass = $plg->p_path.'Plugin'.$plg->p_name;
					if (class_exists($pluginClass,0)) {
						if ($plg->p_enabled){
							Text::parsePlugin($name);
							self::$_loadedPlugins[$name] = new $pluginClass($name,$plg->p_params);
							Portal::getInstance()->addStyleSheet("plugins/".$name.".css");
						} else {
							Text::parsePlugin($name);
							self::$_installedPlugins[$name] = new $pluginClass($name,$plg->p_params, 0);
						}
					} else {
						self::$_corruptedPlugins[$name] = true;
						Debugger::getInstance()->warning("Plugin class not loaded : ".$pluginClass);
					}
				} else {
					self::$_corruptedPlugins[$name] = true;
					Debugger::getInstance()->warning("Plugin not found => ".$pluginScriptPath);
				}
			}
			Debugger::getInstance()->milestone("Plugins loaded");
		}
	}

	public static function isInstalled($path,$name) {
		$pname=$path.".".$name;
		return (array_key_exists($pname,self::$_installedPlugins) || self::isLoaded($pname));
	}
	public static function isLoaded($name) {
		return array_key_exists($name,self::$_loadedPlugins);
	}
	public static function isCorrupted($path,$name) {
		$pname=$path.".".$name;
		return array_key_exists($pname,self::$_corruptedPlugins);
	}
	public static function getInstance($name, $check_all=0) {
		if (self::isLoaded($name)) {
			return self::$_loadedPlugins[$name];
		}	else {
			if ($check_all) {
				if (array_key_exists($name,self::$_installedPlugins)) {
					return self::$_installedPlugins[$name];
				}	else {
					Util::fatalError("Plugin not installed => ".$name);
				}
			} else Util::fatalError("Plugin not loaded => ".$name);
		}
	}
	private function __construct($pluginName,$params="", $force_events=1) {
		$this->initObj($pluginName);
		if ($force_events) $this->registerEvents();
		$this->setDefaultParams($params);
	}
	private function setDefaultParams($params){
		$paramsMask=$this->getParamsMask();
		$pluginName=$this->getName();
		if (count($paramsMask)) foreach($paramsMask as $key=>$val) {
			$this->set($key,$val['vdefault']);
			$this->_loadedConfig[$key] = $val['vdefault'];
		}
		$configArray=Params::parse($params);
		if (count($configArray)) {
			foreach ($configArray as $configKey=>$configVal) {
				if (array_key_exists($configKey,$paramsMask)) {
					$this->set($configKey,$configVal);
					$this->_loadedConfig[$configKey] = $configVal;
				}
			}
		}
		/*
		 if (count($configArray)) {
		foreach ($configArray as $configPairText) {
		if (trim($configPairText) != "") {
		$configPair = explode('=',$configPairText);
		if(isset($configPair[0])) {
		$configKey = $configPair[0];
		if (array_key_exists($configKey,$paramsMask)) {
		if(isset($configPair[1])) $configValue = $configPair[1]; else $configValue = $paramsMask[$configKey]["vdefault"];
		$this->set($configKey,$configValue);
		}
		}
		}
		}
		}
		*/
	}
	public function executeEvent($event,$params=array(),&$data) {
		if (count($this->_loadedConfig)) {
			foreach($this->_loadedConfig as $k=>$v){
				$this->set($k,$v);
			}
		}
		foreach ($params as $key=>$value) {
			$this->set($key,$value);
		}
		return $this->onRaise($event, $data);
	}
	protected function registerEvents(){
		foreach($this->_events as $_event){
			Event::register($_event,$this->getName());
		}
	}
	// Functions for override
	public function getParamsMask(){
		return array();
	}
	protected function onRaise($event, &$data) {
	}
}

?>