<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Router extends Object {
	//---------- Singleton implementation ------------
	private static $_sefMode = false;
	private static $_instance = null;
	private static $_routers = array();
	private static $_vars = array();
	private $_request = "";
	private $_module = "";

	private static function createInstance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
			Debugger::getInstance()->milestone("Created master router");
		}
		return self::$_instance;
	}
	public static function getInstance($moduleName="") {
		if ($moduleName) {
			if (array_key_exists($moduleName, self::$_routers)) { // уже обращались за роутером
				if (self::$_routers[$moduleName]) { // и нашли
					return self::$_routers[$moduleName];
				}	else {	// не существует отдельного роутера
					return self::createInstance();
				}
			} else { // первое обращение к customRouter
				if (self::initCustomRouter($moduleName)) { // создали отдельный роутер
					return self::$_routers[$moduleName];
				}	else { // не существует отдельного роутера
					return self::createInstance();
				}
			}
		} else { // нет имени используем базовый
			return self::createInstance();
		}
	}
	//------------------------------------------------
	private function __construct() {
		if(defined('_ADMIN_MODE')) self::$_sefMode=false;
		else self::$_sefMode=seoConfig::$sefMode;
	}
	private static function initCustomRouter($moduleName) {
		if(defined('_ADMIN_MODE')) $_custom_router_file=PATH_FRONT_MODULES.$moduleName.DS."router.php";
		else $_custom_router_file=PATH_MODULES.$moduleName.DS."router.php";
		if (file_exists($_custom_router_file)) {
			require_once($_custom_router_file);
			$_custom_router_class=$moduleName.'CustomRouter';
			if (class_exists($_custom_router_class,false)) {
				self::$_routers[$moduleName]=new $_custom_router_class;
				Debugger::getInstance()->milestone("Created router for ".$moduleName);
			} else self::$_routers[$moduleName]=false;
		} else self::$_routers[$moduleName]=false;
		return self::$_routers[$moduleName];
	}
	private function checkIntrusion() {
		foreach(self::$_vars as $key=>$value)	{
			$value=Util::checkIntrusion($value);
			$_REQUEST[$key]=$value;
			self::$_vars[$key]=$value;
		}
	}
	public function init()  {
		self::checkManualRedirect();
		if (self::$_sefMode) {
			if(isset($_SERVER['REQUEST_URI'])){
//				if(seoConfig::$strictSefMode&&(preg_match("/index\.php(.*)/",$_SERVER['REQUEST_URI']))) { Util::fatalError(Text::_("Page absent"),"404"); }
				if(!preg_match("/ref_(.*)\.html/",$_SERVER['REQUEST_URI'],$result)) {
					$request=preg_split('/[\?]/',$_SERVER['REQUEST_URI']);
				} else {
					$req="index.php?referral=".$result[0];
					$request=preg_split('/[\?]/',$req);
				}
				$this->_request=$request[0];
				$main_request=preg_split('/[\/]/',$request[0]);
				if (count($main_request)>0) { //getting module
					if (!$main_request[0]) {
						$last_chance=$main_request[1];
					} else {
						$last_chance=$main_request[0];
					}
					$_last_chance=preg_split('/[.]/',$last_chance);
					$this->_module=$_last_chance[0];
				}
				self::$_vars=Router::getInstance($this->_module)->parseRoute($this->_request);
				$this->checkIntrusion();
			}
		} else {
			$this->_module=Request::getSafe("module", "");
			if ($this->_module) {
				$this->setVarsVal("module", $this->_module);
				$view=Request::getSafe("view","");
				if ($view) {
					$this->setVarsVal("view", $view);
					$layout=Request::getSafe("layout","");
					if ($layout) $this->setVarsVal("layout", $layout);
				}
				$psid=Request::getSafe("psid",0); if ($psid) $this->setVarsVal("psid", $psid);
			}
		}
	}
	private function checkManualRedirect(){
		$old_url1=Request::getSafe("REQUEST_URI","","server");
		$old_url=htmlspecialchars_decode($old_url1,ENT_NOQUOTES);
		$referer=siteConfig::$siteUrl;
		$sql="SELECT * FROM #__redirect_links WHERE (rl_old_url='".$old_url."' or rl_old_url='".$old_url1."') AND rl_published=1 AND rl_deleted=0";
		Database::getInstance()->setQuery($sql);
		$new_url=Database::getInstance()->loadObject($object);
		//Debugger::getInstance()->milestone("Redirect link QUERY ".$sql);
		//Debugger::getInstance()->milestone("Redirect link new url(outer)- ".$new_url);
		if (is_object($object) && $object->rl_new_url) Util::redirect($object->rl_new_url,"",301,$referer);
	}
	public function getVarsArr() {
		return self::$_vars;
	}
	public function setVarsVal($name, $val) {
		self::$_vars[$name]=$val;
	}
	public function getRequest($varname) {
		if (array_key_exists($varname, self::$_vars)) return self::$_vars[$varname];
		else return null;
	}
	public function buildRoute($options, $mode="sitelink",$force_fronte=0) {
		if (array_key_exists('alias', $options)) unset($options['alias']);
		switch ($mode) {
			case "nomodule":
				if (array_key_exists('module', $options))  unset($options['module']);
				if (array_key_exists('view', $options))  unset($options['view']);
				if (array_key_exists('layout', $options)) unset($options['layout']);
				$url="default.html"; $appendix="";
				if (count($options)>0) {
					foreach($options as $key=>$val) {
						$appendix.="&".$key."=".$val;
					}
				}
				$url=Portal::getURI($force_fronte).$url;
				if ($appendix) $url=$url."?".substr($appendix,1);
				break;
			case "sitelink":
			default:
				if (array_key_exists('module', $options)) {
					$url = $options['module']; unset($options['module']);
					if (array_key_exists('view', $options)&&$options['view']) {
						$url.= "/".$options['view']; unset($options['view']);
						if (array_key_exists('layout', $options) && $options['layout']) {
							$url.= "/".$options['layout']; unset($options['layout']);
							if (array_key_exists('psid', $options) && $options['psid']) {
								$url.= "/".$options['psid']; unset($options['psid']);
							} else $url.= "/default";
						} elseif (array_key_exists('psid', $options) && $options['psid']) {
							$url.= "/".$options['psid']; unset($options['psid']);
						} else $url.= "/default";
					} else $url.= "/default";
				} else $url.= "default";

				$url.=".html"; $appendix="";
				if (count($options)>0) {
					foreach($options as $key=>$val) {
						$appendix.="&".$key."=".$val;
					}
				}
				$url=Portal::getURI($force_fronte).$url;
				if ($appendix) $url=$url."?".substr($appendix,1);
				break;
		}
		return $url;
	}
	/*
	 * 	default pattern used  'module'=>'' 'view'=>'' 'layout'=>'' 'psid'=>''
	*/
	public function parseRoute($fake) {
		Debugger::getInstance()->milestone("Router (master) parsed URI ".$this->_request);
		$_result=preg_split('/[\/]/',$this->_request);
		$module=""; $view=""; $layout=""; $psid="";
		$last_element = preg_split('/(.html)/',$_result[count($_result)-1]);
		if (preg_match('/(index.php)/',$last_element[0])) $last_element[0]="";
		if (preg_match('/(default)/',$last_element[0])) $last_element[0]="";
		$_result[count($_result)-1] = $last_element[0];
		switch (count($_result)) {
			case 2:
				$module = $_result[1];
				break;
			case 3:
				$module = $_result[1]; $view   = $_result[2];
				break;
			case 4:
				$psid = intval($_result[3]);
				$layout = $_result[3];
				if ($psid) {
					if (strval($psid)===$layout){
						$layout = "";
						// это число
					} else {
						$psid = "";
					}
				} else $psid = "";
				$module = $_result[1]; $view   = $_result[2];
				break;
			case 5:
				$module = $_result[1]; $view = $_result[2];
				$layout = $_result[3]; $psid = intval($_result[4]);
				break;
		}
		$_vars = array();
		if ($module) {
			$_vars['module']=$module;
			if ($view) $_vars['view']=$view;
			if ($layout) $_vars['layout']=$layout;
			if ($psid) $_vars['psid']=$psid;
		}
		return $_vars;
	}
	public static function isJavaScript($_url)  {
		if (preg_match('/(javascript:)/',$_url)) return true;
		else return false;
	}
	public static function isFullLink($_url)  {
		if (preg_match('/(http:\/\/)/',$_url)) return true;
		elseif (preg_match('/(https:\/\/)/',$_url)) return true;
		elseif (preg_match('/(ftp:\/\/)/',$_url)) return true;
		else return false;
	}
	public static function isAnchor($_url)  {
		if (substr($_url, 0,1)=="#") return true;
		else return false;
	}
	public static function _($_url, $sefModeOverride=false, $HtmlSpecialChars=true) {
		if (self::isFullLink($_url)) return $_url;
		if (self::isJavaScript($_url)) return $_url;
		if (self::isAnchor($_url)) return $_url;
		$url = str_replace("&amp;", "&", $_url);
		if ($url=="index.php") return Portal::getURI();
		if (($sefModeOverride)||(self::$_sefMode)) { // sefMode=true
			$force_fronte=1;
			$options=array();
			$parts=preg_split('/(\?)/',$url);
			if ($parts[0]!="index.php") {
				$url = Portal::getURI().$url;
			} // no index.php in URI
			else { // begin parsing URI
				$subparts=preg_split('/(\&)/',$parts[1]);
				if (count($subparts)>0) {
					foreach ($subparts as $subpart) {
						if ($subpart) {
							$option=preg_split('/(\=)/',$subpart);
							$options[$option[0]]=$option[1];
						}
					}
				}
				if (array_key_exists('module',$options)) $module_name=$options['module']; // found module
				else $module_name="nomodule";
				$url=Router::getInstance($module_name)->buildRoute($options,$module_name,$force_fronte);
			}
		} else { $url=Portal::getURI().$url; }
		if ($HtmlSpecialChars) $url = str_replace("&", "&amp;", $url);
		return $url;
	}
}
?>