<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class SearchMachine extends Object {

	//---------- Singleton implementation ------------
	private static $_instance = null;

	private $whereInputs=array();
	private $defSorting=array();
	private $sorting=array();
	private $orderBy="ttl";
	private $keywords=array();
	private $stype="any";
	private $swhere="";
	private $page=1;
	private $link="index.php?task=search";
	private $pagesize=3;
	private $view=null;
	private $paginator=null;
	
	public static function createInstance($critFail=true) {
		if (self::$_instance == null) {
			self::$_instance = new self($critFail);
		}
	}
	public static function getInstance($critFail=true) {
		self::createInstance($critFail);
		return self::$_instance;
	}
	public function addWhereInputs($inputs){
		if (count($inputs)){
			foreach ($inputs as $fld=>$text){
				$this->whereInputs[$fld]=$text;
			}
		}
	}
/*
	public function addWhereParams($inputs){
		if (count($inputs)){
			foreach ($inputs as $fld=>$text){
				$this->whereParams[$fld]=$text;
			}
		}
	}
	public function getWhereParams(){
		return $this->whereParams;
	}
*/
	public function getWhereInputs(){
		return $this->whereInputs;
	}
	
	public function getWhere(){
		return $this->swhere;
	}

	public function getType(){
		return $this->stype;
	}

	public function getLink(){
		return $this->link;
	}
	
	public function getPageSize(){
		return $this->pagesize;
	}
	
	public function getWords(){
		return $this->keywords;
	}
	public function addSorting($arrs){
		if (count($arrs)){
			foreach ($arrs as $fld=>$text){
				$this->sorting[$fld]=$text;
			}
		}
	}
	
	public function addDefaultSorting($arrs){
		if (count($arrs)){
			foreach ($arrs as $fld=>$text){
				$this->defSorting[$fld]=$text;
			}
		}
	}
	public function getOrderBy(){
		if (in_array($this->orderBy, $this->sorting[$this->getWhere()])) return str_replace("-", " ", $this->orderBy);
		else return str_replace("-", " ", $this->defSorting[$this->getWhere()][0]);
	}
	
	public function getOrderByNR(){
		if (in_array($this->orderBy, $this->sorting[$this->getWhere()])) return $this->orderBy;
		else return $this->defSorting[$this->getWhere()][0];
	}
	
	public function parseRequest(){
		$this->pagesize=Request::get("rpp",siteConfig::$recordsPerPage);
		$this->stype=Request::getSafe("searchtype","any"); 
		$keywords = trim(Request::getSafe("keywords",""));
		if (!$keywords) $keywords=trim(urldecode(Request::getSafe("kwds","")));
		$kwds=explode(" ",$keywords);
		if (count($kwds)){
			foreach($kwds as $kw) {
				$kw=trim($kw);		if ($kw) $this->keywords[]=$kw;
			}
		}
		$this->swhere=Request::getSafe("where_search",false);
		$this->page=Request::getSafe("page",1);
		$this->orderBy=Request::getSafe("orderby","");
		$this->link="index.php?task=search&amp;searchtype=".$this->stype
							."&amp;where_search=".$this->swhere
							."&amp;kwds=".urlencode(implode(" ",$kwds));
		
		
	}
	
	public function renderForm(){
		$module="";
		$view="";
		$task="search";
		echo "<h3 class=\"title\">".Text::_("Search in site")."</h3>";
		echo "<form action=\"".Router::_("index.php")."\" method=\"post\" name=\"frmSearch\">";
		echo "<div class=\"searchform\">";
		echo HTMLControls::renderLabelField("keywords","Search phrase",1);
		echo HTMLControls::renderInputText("keywords",implode(" ",$this->keywords),80);
		echo "</div>";
		echo "<div class=\"searchform_radio\">";
		echo HTMLControls::renderRadio("searchtype", "searchtype_any", "any",Text::_("Any word"),"", $this->stype=="any");
		echo HTMLControls::renderRadio("searchtype", "searchtype_all", "all",Text::_("All words"),"", $this->stype=="all");
		echo HTMLControls::renderRadio("searchtype", "searchtype_exact", "exact",Text::_("Exact phrase"),"", $this->stype=="exact");
		echo "</div>";
		$inputs=$this->getWhereInputs();
		if (count($inputs)) {
			echo "<div class=\"searchform_selector\">";
			echo HTMLControls::renderSelect("where_search","" , "", "", $inputs,$this->swhere,0);
			echo "</div>";
		}
		echo "<div class=\"searchform_button\">";
		echo HTMLControls::renderButton("submit", $val=Text::_("Start search"),  "submit");
		echo HTMLControls::renderHiddenField("module", $module);
		echo HTMLControls::renderHiddenField("view", $view);
		echo HTMLControls::renderHiddenField("task", $task);
		echo "</div>";
		echo "</form>";
	}
	public function renderResult() {
		if (!count($this->keywords)) return false;
  	$this->view=new View("SearchMachine");
		Event::raise("search.renderResult",array(),$data);
		$filename=PATH_TEMPLATES.SiteConfig::$siteTemplate.DS."search.php";
		if (file_exists($filename)) {
			require_once($filename);
		} else {
			echo "<hr />";
			if (count($data)) {
				echo $this->renderSortPanel(); 
				foreach($data as $row){
					echo "<div class=\"searchRowTitle\"><a rel=\"nofollow\" href=\"".$row["link"]."\" class=\"articleTitle\">".$row["ttl"]."</a></div>";
					if ($row["cdate"]) {
						echo "<div class=\"searchRowDate\">".$row["cdate"]."</div>";
					}
					if ($row["img"]) {
						echo "<div class=\"searchRowShort\">";
						echo $row["img"];
						echo "</div>";
					}
					echo "<div class=\"searchRowShort\">";
					echo $row["txt"];
					echo "</div>";
				}
			} else { echo "<p class=\"searchResult\">".Text::_("Records not found")."</p>"; }
		}
		echo $this->view->renderPagePanel(); 
	}
	public function alterPaginator($resultsCount=0) {
  	$this->paginator=new Paginator($this->view, $resultsCount, $this->pagesize);
  	$this->paginator->buildPagePanel($this->link, "");
  	return $this->paginator->getAppendix();
	}	
	public function renderSortPanel(){
		if (count($this->sorting)>1){
			echo "<div class=\"searchSort\">";
			$activeSort=$this->getOrderByNR();
			$baselink=$this->getLink()."&amp;page=".$this->page;
			foreach($this->sorting[$this->getWhere()] as $key=>$sort){
				if ($activeSort==$sort) $class="class=\"active\""; else $class="";
				echo "<a ".$class." href=\"".$baselink."&amp;orderby=".$sort."\">".Text::_($sort)."</a>";
			}
			echo "</div>";
		}
	}
}
?>