<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

final class User extends Object {

	//---------- Singleton implementation ------------
	private static $_instance = null;

	public static function createInstance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
			self::$_instance->adjustACL();
			Debugger::getInstance()->milestone("User created");
		}
	}

	public static function getInstance() {
		self::createInstance();
		return self::$_instance;
	}
	//------------------------------------------------

	private $_loggedIn	= false;

	public $u_id		= 0;
	public $u_login		= '';
	public $u_nickname	= '';
	public $u_email		= '';
	public $u_status	= 0;
	public $u_lastvisit	= '';
	public $u_role		= 3; // 3 это гость
	public $u_referral	= "";
	public $u_account	= 0;
	public $u_points	= 0;
	public $u_discount	= 0;
	public $u_pricetype	= 1;
	public $u_rating	= 0;
	public $u_source	= "";
	public $u_last_visit = "0000-00-00 00:00:00";
	
	private $_isAdmin				= false;
	private $_canMaintenanceLogin	= false;

	private	$_friends	= null;

	private function __construct() {
		$this->initObj();
		$db = Database::getInstance();
		$uid = intval($this->get('user_id',0));
		if ($uid != 0) {
			// Load user data
			$query = "SELECT u.*, ar.ar_admin FROM #__users AS u, #__acl_roles AS ar WHERE ar.ar_id=u.u_role AND u.u_id=".$uid;
			$db->setQuery($query);
			$db->loadObject($this);
			if($this->u_id){
				$this->_isAdmin = (1 == intval($this->ar_admin));
				$this->_loggedIn = true;
				if (!$this->u_email){ // здесь если не в кабинете и не выходим то постоянно перенаправляем на заполнение мыла
					if (Request::getSafe("option")!="logout" && Request::getSafe("module")!="user") Util::redirect(Router::_("index.php?module=user&view=reset&layout=email"));
				}
			}
		} else {
			$referral = strval(Request::get('referral',''));
			if ($referral) {
				if (isset($_COOKIE['referral'])) {
					$old_referral=$_COOKIE['referral'];
					if (!$old_referral) setcookie('referral', $referral, time()+7776000);
				} else setcookie('referral', $referral, time()+7776000);
			}
			$query = "SELECT `ar_id` FROM `#__acl_roles` WHERE `ar_name`='guest'";
			$db->setQuery($query);
			$this->_isAdmin = false;
			$this->u_role = intval($db->loadResult());
		}
		Event::raise('user.authenticate');
	}

	private function adjustACL() {
		if (!defined('_ADMIN_MODE')) {
			$this->_canMaintenanceLogin = ACLObject::getInstance('maintenanceLogin')->canAccess();
		}	else {
			$this->_canMaintenanceLogin = true;
		}
		if (siteConfig::$siteDisabled && !$this->_canMaintenanceLogin && $this->_loggedIn) {	$this->logout(); }
	}
	public function getIP() {
		return Session::getInstance()->getIP();
	}
	public function getAgent() {
		return Session::getInstance()->getAgent();
	}
	public function login($login,$password,$remember) {
		Event::raise('user.before_login',array());
		if ($login == '' || $password == '') Util::redirect($this->get('returnUrl'),Text::_('Login error'));
		if((User::checkFloodPoint())) Util::redirect($this->get('returnUrl'),Text::_("Flood found"));
		$db = Database::getInstance();
		if(backofficeConfig::$allowEmailLogin) $login=self::getLoginForEmail($login);
		$return_url=$this->get('returnUrl');
		if(!$login) Util::redirect($return_url,Text::_('Wrong Username or password'));
		$secret=$this->encodePassword($login,$password);
		$query = "SELECT * FROM #__users WHERE u_source='system' AND u_deleted=0 AND u_login='".$login."' AND u_secret='".$secret."'";
		$db->setQuery($query);

		$usr = new stdClass();
		if (!$db->loadObject($usr)) {
//			$return_url=$this->get('returnUrl');
			$server_uri = "http://".$_SERVER['SERVER_NAME']."/";
			if (preg_match("/administrator/", $return_url)) $return_url=$server_uri."index.php?module=user&task=login&return_url=".base64_encode($server_uri."administrator/index.php");
			Util::redirect($return_url,Text::_('Wrong Username or password'));
		}
		if ((intval($usr->u_activated) != 1)||(intval($usr->u_role) == 0)) {
			Util::redirect($this->get('returnUrl'),Text::_('User not activated or role not present'));
		}
		$ip=User::getInstance()->getIP();
		$msgs=$this->inBlackList($login,$usr->u_nickname,$ip,$usr->u_email);
		if (count($msgs)) {
			Util::redirect($this->get('returnUrl'),implode(",", $msgs));
		}
		if (siteConfig::$siteDisabled) {
			$this->u_id = $usr->u_id;
			$this->u_role = $usr->u_role;
			ACLObject::clearACL();
			if (!ACLObject::getInstance("maintenanceLogin", false)->canAccess()) {
				Util::redirect('index.php',Text::_('Site is disabled'));
			}
		}
		Session::getInstance()->create($usr->u_id, $remember);
		$this->updateLastVisit($usr->u_id);
		Event::raise('user.after_login',array("usr"=>$usr));
		$return_to=$this->get('returnUrl');
		if (preg_match("/logout/",$return_to)) Util::redirect("index.php");
		else Util::redirect($this->get('returnUrl'));
	}
	public function elogin($login,$source) {
		Event::raise('user.before_login',array());
		if ($login == '' || $source == '' || $source == 'system') Util::redirect($this->get('returnUrl'),Text::_('Login error'));
		$db = Database::getInstance();
		$query = "SELECT * FROM #__users WHERE u_source='".$source."' AND u_deleted=0 AND u_login='".$login."'";
		$db->setQuery($query);

		$usr = new stdClass();
		if (!$db->loadObject($usr)) {
			$return_url=$this->get('returnUrl');
			$server_uri = "http://".$_SERVER['SERVER_NAME']."/";
			if (preg_match("/administrator/", $return_url)) $return_url=$server_uri."index.php?module=user&task=login&return_url=".base64_encode($server_uri."administrator/index.php");
			Util::redirect($return_url,Text::_('Wrong Username or password'));
		}
		if ((intval($usr->u_activated) != 1)||(intval($usr->u_role) == 0)) {
			Util::redirect($this->get('returnUrl'),Text::_('User not activated or role not present'));
		}
		$ip=User::getInstance()->getIP();
		$msgs=$this->inBlackList($login,$usr->u_nickname,$ip,$usr->u_email);
		if (count($msgs)) {
			Util::redirect($this->get('returnUrl'),implode(",", $msgs));
		}
		if (siteConfig::$siteDisabled) {
			$this->u_id = $usr->u_id;
			$this->u_role = $usr->u_role;
			ACLObject::clearACL();
			if (!ACLObject::getInstance("maintenanceLogin", false)->canAccess()) {
				Util::redirect('index.php',Text::_('Site is disabled'));
			}
		}
		Session::getInstance()->create($usr->u_id, 0);
		Event::raise('user.after_login',array("usr"=>$usr));
		if (!$usr->u_email){
			Util::redirect(Router::_("index.php?module=user&view=reset&layout=email&return_url=".base64_encode($this->get('returnUrl'))));
		}	else Util::redirect($this->get('returnUrl'));
	}
	public function inBlackList($login,$nickname,$ip,$email){
		$msg=array();
		$db=Database::getInstance();
		$sql="SELECT * FROM #__blacklist";
		$sql.=" WHERE bl_enabled=1 AND bl_deleted=0";
		$sql.=" AND((bl_val='".$login."' AND bl_type='login')";
		$sql.=" OR (bl_val='".$ip."' AND bl_type='ip')";
		$sql.=" OR (bl_val='".$nickname."' AND bl_type='nickname')";
		$sql.=" OR (bl_val='".$email."' AND bl_type='email'))";
		$db->setQuery($sql);
		$res=$db->loadObjectList();
		if (count($res)){
			foreach($res as $ibl){
				$msg[]=Text::_($ibl->bl_type." in blacklist");
			}
		}
		return $msg;
	}
	public function logout() {
		if ($this->_loggedIn) {
			Session::getInstance()->destroy();
		}

		Event::raise('user.logout');
		Util::redirect($this->get('returnUrl'));
	}


	public function isLoggedIn() {
		return $this->_loggedIn;
	}

	public function getID() {
		return $this->u_id;
	}

	public function getStatusID() {
		return $this->u_status;
	}
	public function getLogin() {
		return $this->u_login;
	}
	
	public function getNickname() {
		return $this->u_nickname;
	}

	public function getEmail() {
		return $this->u_email;
	}

	public function getProvider() {
		return $this->u_source;
	}

	public function isAdmin() {
		return $this->_isAdmin;
	}

	public static function getNicknameFor($uid) {
		$db = Database::getInstance();
		$query = "SELECT u_nickname FROM #__users WHERE u_id=".(int)$uid;
		$db->setQuery($query);
		return $db->loadResult();
	}

	public static function getEmailFor($uid) {
		$db = Database::getInstance();
		$query = "SELECT u_email FROM #__users WHERE u_id=".(int)$uid;
		$db->setQuery($query);
		return $db->loadResult();
	}

	public function getIdFor($nickname) {
		$db = Database::getInstance();
		$query = "SELECT u_id FROM #__users WHERE u_nickname='".$nickname."'";
		$db->setQuery($query);
		return intval($db->loadResult());
	}
	public function getInfoUserFor($nickname) {
		$db = Database::getInstance();
		$query = "SELECT u_id, u_email FROM #__users WHERE u_nickname='".$nickname."'";
		$db->setQuery($query);
		$res=false;
		$db->loadobject($res);
		return $res;
	}
	public static function getUserId($login) {
		$db = Database::getInstance();
		$query = "SELECT u_id FROM #__users WHERE u_login='".$login."'";
		$db->setQuery($query);
		return intval($db->loadResult());
	}
	
	public static function getUserIdByMail($email) {
		$db = Database::getInstance();
		$query = "SELECT u_id FROM #__users WHERE u_email='".$email."'";
		$db->setQuery($query);
		return intval($db->loadResult());
	}
	
	
	public static function getLoginFor($uid) {
		$db = Database::getInstance();
		$query = "SELECT u_login FROM #__users WHERE u_id=".(int)$uid;
		$db->setQuery($query);
		return $db->loadResult();
	}
	private static function getLoginForEmail($email) {
		$db = Database::getInstance();
		$query = "SELECT u_login FROM #__users WHERE u_email='".$email."'";
		$db->setQuery($query);
		return $db->loadResult();
	}
	public static function getIDByAffiliate($affiliate){
		$db = Database::getInstance();
		$query = "SELECT u_id FROM #__users WHERE u_affiliate_code='".$affiliate."'";
		$db->setQuery($query);
		return $db->loadResult();
	}
	public function getRole() {
		return $this->u_role;
	}
	public function getLastVisit() {
		return $this->u_last_visit;
	}
	public function getRoleName() {
		$db = Database::getInstance();
		$query = "SELECT ar_name FROM #__acl_roles WHERE ar_id=".($this->u_role);
		$db->setQuery($query);
		$role_name = $db->loadResult();
		return $role_name;
	}

	public static function isUser($login,$psid=0) {
		$db = Database::getInstance();
		$query = "SELECT COUNT(*) FROM #__users WHERE u_login='".strval($login)."'";
		if ($psid) $query.=" AND u_id<>".$psid;
		$db->setQuery($query);
		$cnt = $db->loadResult();
		return (intval($cnt) != 0);
	}

	public static function isEmail($mail,$psid=0) {
		$db = Database::getInstance();
		$query = "SELECT COUNT(*) FROM #__users WHERE u_email='".strval($mail)."'";
		if ($psid) $query.=" AND u_id<>".$psid;
		$db->setQuery($query);
		$cnt = $db->loadResult();
		return (intval($cnt) != 0);
	}
	public static function isNickName($nickname,$psid=0) {
		$db = Database::getInstance();
		$query = "SELECT COUNT(*) FROM #__users WHERE u_nickname='".strval($nickname)."'";
		if ($psid) $query.=" AND u_id<>".$psid;
		$db->setQuery($query);
		$cnt = $db->loadResult();
		return (intval($cnt) != 0);
	}
	
	public static function getReferral($affcode) {
		$db = Database::getInstance();

		$query = "SELECT `u_id` FROM `#__users` WHERE `u_affiliate_code`='".$affcode."'";
		$db->setQuery($query);
		return intval($db->loadResult());
	}

	public static function getNicknameByAffCode($affcode) {
		$db = Database::getInstance();

		$query = "SELECT `u_nickname` FROM `#__users` WHERE `u_affiliate_code`='".$affcode."'";
		$db->setQuery($query);
		return strval($db->loadResult());
	}

	public static function genAffiliateCode($uid,$length=12) {
		$heap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$ac = '';

		$length -= strlen(strval($uid));
		for ($i = 0; $i < $length; $i++) {
			$ri = rand(0,strlen($heap)-1);
			$ac .= $heap[$ri];
		}

		return strtoupper($ac.strval($uid));
	}
	
	public static function genValidationCode($uid,$length=25) {
		$heap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$ac = '';

		$length -= strlen(strval($uid));
		for ($i = 0; $i < $length; $i++) {
			$ri = rand(0,strlen($heap)-1);
			$ac .= $heap[$ri];
		}

		$vcode=strtoupper($ac.strval($uid));
		$db= Database::getInstance();
		$sql="UPDATE #__users set u_validation='".$vcode."' WHERE  u_deleted=0 AND u_id=".(int)$uid;
		$db->setQuery($sql);
		$db->query();
		return $vcode;
	}
	
	
	public static function addUser($login,$nickname,$password,$email,$role,$referral="",$activated,$Discount=0,$priceType=1,$provider="system") {
		$db = Database::getInstance();
    $secret=User::getInstance()->encodePassword($login,$password);
		$affiliateCode = 'tempcode';
		$accountBonus=(int)siteConfig::$def_account_bonus;
		$query="INSERT INTO #__users 
							(u_id,u_affiliate_code,u_referral,u_login,
							u_secret,u_email,u_reg_date,u_nickname,
							u_account,u_points,u_discount,u_pricetype,
							u_role,u_rating,u_activated,u_deleted,u_source)
						VALUES(
							NULL,'".$affiliateCode."','".$referral."',".DBUtil::quote($login).",'"
							.$secret."',".DBUtil::quote($email).",NOW(),".DBUtil::quote($nickname).","
							.$accountBonus.",0,".$Discount.",".$priceType.","
							.$role.",0,0,0,'".$provider."')";
		
		$db->setQuery($query);
		if (!$db->query()) return false;
		$uid = $db->insertid();
		Profile::addProfile($uid);
		$affiliateCode = self::genAffiliateCode($uid);
		$query = "UPDATE `#__users` SET `u_affiliate_code`='".$affiliateCode."',`u_activated`='".$activated."' WHERE `u_id`=".$uid;
		$db->setQuery($query);
		if (!$db->query()) return false;
		Event::raise("user.register",array("userLogin"=>$login,"userEmail"=>$email));
		return $uid;
	}

	public static function saveUser($uid,$login,$nickname,$email,$role,$activated,$Discount=0,$priceType=1) {
		$db = Database::getInstance();
		$query = "UPDATE #__users SET u_login=".DBUtil::quote($login);
		$query.= ", u_nickname=".DBUtil::quote($nickname);
		if (defined("_ADMIN_MODE")) $query.= ", u_discount=".$Discount.", u_pricetype=".intval($priceType);
		$query.= ", u_email=".DBUtil::quote($email).", u_role=".intval($role);
		$query.= ", u_activated=".$activated."  WHERE u_id=".intval($uid);

		$db->setQuery($query);
		return $db->query();
	}

	public static function setPassword($uid,$password,$login) {
		$db = Database::getInstance();
    $secret=User::getInstance()->encodePassword($login,$password);
		$query = "UPDATE #__users SET u_secret='".$secret."' WHERE u_id=".intval($uid);
		$db->setQuery($query);
		$db->query();
	}
	public function resetEmail($email){
		$db = Database::getInstance();
		$uid=User::getInstance()->getID();
		if ($uid && !User::isEmail($email)) {
			$query = "UPDATE #__users SET u_email='".$email."' WHERE u_id=".intval($uid);
			$db->setQuery($query);
			return $db->query();
    } else return false;
	}
	public function resetPassword($login,$password,$vcode) {
		$db = Database::getInstance();
		// возможно что смена пароля идет прямо из кабинета
    if (User::getInstance()->isLoggedIn()) $uid=User::getInstance()->getID();
    else { // а возможно не авторизовался но пришел по ссылке
			$sql_test = "SELECT u_id FROM #__users WHERE u_deleted=0 AND u_login='".$login."' AND u_validation='".$vcode."'";
  	  $db->setQuery($sql_test);
    	$uid=$db->loadResult();
    }
    if ($uid) {
	    $secret=$this->encodePassword($login,$password);
    	$query = "UPDATE #__users SET u_validation='', u_secret='".$secret."' WHERE u_id=".intval($uid);
			$db->setQuery($query);
			return $db->query();
    } else return false;
	}
	
	public static function setActivation($uid,$activated) {
		$db = Database::getInstance();
		$query = "UPDATE `#__users` SET `u_activated`=".intval($activated)." WHERE `u_id`=".intval($uid);
		$db->setQuery($query);
		$db->query();
	}

	public static function deleteUser($uid) {
		$db = Database::getInstance();

		$query = "UPDATE `#__users` SET `u_deleted`=1 WHERE `u_id`=".intval($uid);
		$db->setQuery($query);
		$db->query();

		/*
		 $query = "UPDATE `#__profiles` SET `pf_deleted`=1 WHERE `pf_user_id`=".intval($uid);
		 $this->_db->setQuery($query);
		 $this->_db->query();
		 */
	}

	public static function getDefaultRole($name='') {
		$db = Database::getInstance();
		if($name){
			$query = "SELECT ar_id FROM #__acl_roles WHERE ar_name='".$name."'";
			$db->setQuery($query);
			return intval($db->loadResult());
		} else {
			$query="SELECT ar_id FROM #__acl_roles WHERE ar_id=".(int)backofficeConfig::$defaultUserRole;
			$db->setQuery($query);
			//echo $db->getQuery();
			if($db->LoadResult()) return (int)backofficeConfig::$defaultUserRole;
			else return 0;
		}
	}

	public static function checkReferral($affcode=0) {
		if(!$affcode)	if (isset($_COOKIE['referral']))  $affcode=$_COOKIE['referral']; else $affcode=0;
		if ($affcode) {
			$db = Database::getInstance();
			$query = "SELECT count(u_id) FROM `#__users` WHERE `u_affiliate_code`='".$affcode."'";
			$db->setQuery($query);
			if($db->LoadResult()>0) return 	$affcode;
			else return 0;
		} else return 0;
	}

	/** возвращает true если флуд **/
	public static function checkFloodPoint($delay=0){
		if(!$delay) $delay=backofficeConfig::$floodDelay;
		if (isset($_SESSION['flood_point'])) {
			if (($_SESSION['flood_point']+$delay) > microtime(1)) {
				return true;
			} else {
				self::setFloodPoint();
				return false;
			}
		} else {
			self::setFloodPoint();
			return false;
		}
	}
	public static function setFloodPoint(){
		$_SESSION['flood_point']=microtime(1);
	}
	public function encodePassword($login,$secret){
		return md5($secret.$login.DatabaseConfig::$dbSecret);
	}
	public static function resetEAuth($all=true){
		if ($all) {
			Session::unsetVar("eauth_type");
			Session::unsetVar("eauth_return");
		}
		Session::unsetVar("eauth_token");
		Session::unsetVar("eauth_token_secret");
	}
	public function increaseAccount($sum,$znak="+")	{
		$sql="update #__users set u_account=u_account".$znak.$sum ." WHERE u_id=".$this->u_id;
		Database::getInstance()->setQuery($sql);
		return  Database::getInstance()->query();
	}
	public function decreaseAccount($sum)	{
		return $this->increaseAccount($sum,"-");
	}
	public function increasePoints($sum,$znak="+") {
		$sql="update #__users set u_points=u_points".$znak.$sum ." WHERE u_id=".$this->u_id;
		Database::getInstance()->setQuery($sql);
		return  Database::getInstance()->query();
	}
	public function decreasePoints($sum) {
		return $this->increasePoints($sum,"-");
	}
	public function trasferAccount2Points($sum) {
		$sql="update #__users set u_points=u_points+".$sum.",u_account=u_account-".$sum." WHERE u_id=".$this->u_id;
		Database::getInstance()->setQuery($sql);
		return  Database::getInstance()->query();
	}
	public function updateLastVisit($uid=0){
		$sql="update #__users set u_last_visit=u_login_date,u_login_date=NOW() WHERE u_id=".$uid;
		Database::getInstance()->setQuery($sql);
		return  Database::getInstance()->query();
		
	}
}
?>