<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

final class Util {
	//---------- PHP locales -------------
	private static $locales = array('en'=>'en_GB', 'ru'=>'ru_RU');
	private static $sortField='';
	private static $sortDir=true;
	public static function getLocale($language='en') {
		$locale = self::$locales['en'];
		if (array_key_exists($language,self::$locales))
			$locale = self::$locales[$language];
		return $locale;
	}
	public static function halt($msg="") {
		die($msg); // это для замены die() в коде
	}
	public static function setupLocale() {
		setlocale( LC_ALL, self::getLocale(siteConfig::$defaultLanguage));
	}
	//---------- Server settings ----------
	public static function checkServerSettings($critical=false) {
		$settings_ok = true; $msg="";
//		if (intval(ini_get('zend.ze1_compatibility_mode')) == 1) { $settings_ok = false; $msg=""; } // Удалена в версии 5.3
		if (intval(ini_get('register_globals')) == 1) { $settings_ok = false; $msg="CMS required register_globals to be disabled.<br />"; }
		if (version_compare(phpversion(), '5.3.0', '<') == true) { $settings_ok = false; $msg="CMS required PHP version >=5.3<br />"; }
		if (!$settings_ok && $critical) Util::halt("Invalid server settings.<br />".$msg);
		return $settings_ok;
	}
		//--------- Debug routines ------------
	public static function fatalError($message,$codeerror="503") {
		@ob_end_clean();
		switch($codeerror){
			case '503':
				self::sendHeader(503);
				ErrorPages::render503($message);
				break;
			case '404':
				self::sendHeader(404);
				ErrorPages::render404("/",$message);
	   			break;
		}
	}
	//-------- Intrusion detection --------
	public static function checkIntrusion($queryString=false) {
		// проверяем входящую строку или то что пришло в запросе после знака вопроса
		if (!$queryString) {
			// запрещаем ссылки вида http://сайт/?
			if(preg_match("/^\/\?(.*)/",$_SERVER['REQUEST_URI'])){ self::fatalError(Text::_("Page absent"),"404"); }
			// забираем весь QUERY_STRING для дальнейшей проверки	
			$queryString = strtolower($_SERVER['QUERY_STRING']);
		}
		// проверяем встречаемость 5 подряд символов
		if (preg_match("/([OdWo5NIbpuU4V2iJT0n]{5}) /", rawurldecode($loc=$queryString), $matches)) { self::fatalError("", "403"); }
		if ((stristr($queryString,'%20union%20'))
		||(stristr($queryString,'/*'))
		||(stristr($queryString,'*/union/*'))
		||(stristr($queryString,'c2nyaxb0'))
		||(stristr($queryString,'+union+'))
		||(stristr($queryString,'http://'))
		||((stristr($queryString,'cmd=')) && (!stristr($queryString,'&cmd')))
		||((stristr($queryString,'exec')) && (!stristr($queryString,'execu')))
		||(stristr($queryString,'concat'))) {
//			self::fatalError(Text::_("Page absent"),"404");
			header("Location: index.php");
		}
		return $queryString;
	}
	//--------- Array management ----------
	public static function getArrParam($arr,$name,$def=null) {
		static $noHtmlFilter 	= null;
		static $safeHtmlFilter 	= null;
		$return = null;
		if (is_array($arr)) {
			if (isset($arr[$name])) $return = $arr[$name];
		}	else if(is_object($arr)) {
			if (property_exists($arr,$name)) $return = $arr->$name;
		}	else return $def;
		if($return == null) return $def;
		if (is_string($return)) $return = trim($return);
		// account for magic quotes setting
//		if (!get_magic_quotes_gpc()) {
//			if(is_string($return))	$return = addslashes($return);
//		}
		return $return;
	}
	public static function logFile($message) {
		$f=fopen(PATH_TMP."debug.log","a");
		if(is_array($message)) $message=json_encode($message);
		$logmessage = Date::nowSQL()." => ".$message.CR_LF."-------".CR_LF;
		fwrite($f,$logmessage);
		fclose($f);
	}
	public static function redirect($url, $message='', $code=302, $referer="") {
		// Debugger редиректов. НЕ УДАЛЯТЬ !!!
		//if(siteConfig::$debugMode) Util::logFile($url.CR_LF.$message.CR_LF.$code.CR_LF.$referer);
		if ($message) $_SESSION['altadoc_message']=$message;
		else $_SESSION['altadoc_message']="";
		$_sent=headers_sent();
		if ((siteConfig::$debugMode && User::getInstance()->isAdmin()) || siteConfig::$debugMode>100){
			ErrorPages::renderRedirect($url, $message, $code);
		}	else 	{
			@ob_end_clean(); // clear output buffer
			if ($_sent) {
				echo "<script>document.location.href='$url';</script>\n";
			}	else {
				$_SESSION['altadoc_redirect_code']=$code;
				switch($code){
					case 404:
						self::sendHeader(404);
						if (seoConfig::$stop404) ErrorPages::render404($url, $message);
						else echo "<script>document.location.href='$url';</script>\n";
						break;
					case 301:
						self::sendHeader(301);
						header("Location: ".$url,true, 301);
					case 302:
					default:
						header("Location: ".$url);
						break;
				}
			}
		}
		self::halt();
	}
	public static function download($filepath,$filename,$redirect="index.php",$msg="") {
		if (headers_sent()) {
			Util::redirect($redirect,$msg);
		}	elseif(!is_file($filepath.$filename)) {
			Util::redirect($redirect,Text::_("File absent"));
		}	else {
			@ob_end_clean(); // clear output buffer
			header ("Content-Type: application/octet-stream");
			header ("Accept-Ranges: bytes");
			header ("Content-Length: ".filesize($filepath.$filename));
			header ("Content-Disposition: attachment; filename=".$filename);
			readfile($filepath.$filename);
			self::halt();
		}
	}
	public static function getReturnUrl($encode=true) {
		if(isset($_SERVER['REQUEST_URI'])) $returnUrl = $_SERVER['REQUEST_URI'];
		else $returnUrl="index.php";
		if ($encode) $returnUrl = base64_encode($returnUrl);
		return $returnUrl;
	}
	public static function getRefererUrl($encode=true) {
		if(isset($_SERVER['HTTP_REFERER'])) $returnUrl = $_SERVER['HTTP_REFERER'];
		else $returnUrl="index.php";
		if ($encode) $returnUrl = base64_encode($returnUrl);
		return $returnUrl;
	}
	public static function showCollapsedArray($array, $name='', $force_object=false, &$iter=0, $flag_head=true) {
		Util::showArray($array, $name, $force_object, $iter, $flag_head, true);
	}
	public static function showArray($array, $name="", $force_object=false, &$iter=0, $flag_head=true, $collapsed=false) {
		if (!$name && $name!='0') {
			if (is_object($array)) $name=get_class($array)."_".rand(0,1000);
			elseif (is_array($array)) $name="no_name_array_".rand(0,1000);
			else $name = "no_name_".rand(0,1000);
		}

		if (!is_array($array) && !is_object($array)) {
			echo "<div id=\"".$iter."_head_".$name."\" class=\"arr_dbg_head\">".$array." is not valid by by this debug function for <span class=\"underline\">".$name."</span></div>";
			return;
		}
		$iter++;
		if ($flag_head) echo "<div id=\"head_".$iter."_".$name."\" class=\"arr_dbg_head\" onclick=\"$('#".$name."_".$iter."').toggle();\">".$name." : ".gettype($array)."</div>";
		if($collapsed) echo "<div id=\"".$name."_".$iter."\" class=\"arr_dbg\" style=\"display:none;\">";
		else echo "<div id=\"".$name."_".$iter."\" class=\"arr_dbg\">";
		if (is_object($array)) {
			if (get_class($array) == 'FieldSet' || is_subclass_of($array,'FieldSet')) {
				$array_vars = $array->toArray();
			}	else {
				if (method_exists($array, "getObjectVars")) $array_vars=$array->getObjectVars();
				else $array_vars = get_object_vars($array);
			}
		} else $array_vars=$array;
		foreach ($array_vars as $index=>$value) {
			echo "<table>";
			if (is_array($value) || is_object($value)) {
				$_iter = $iter + 1;
				echo "<tr><td colspan=\"3\"><fieldset><legend	onclick=\"$('#sub_".$index."_".$_iter."').toggle();\">".$index." : ".gettype($value)."</legend>";
				self::showArray($value, "sub_".$index, $force_object, $iter, false);
				echo "</fieldset></td></tr>";
			}	else echo "<tr><td width=\"20%\">[".$index."]</td><td width=\"3%\"> ====> </td><td width=\"77%\"><b>".$value."</b> <i><small>(typeof ".gettype($value).")</small></i></td></tr>";
			echo "</table>";
		}
		echo "</div>";
	}
	public static function ddump($var, $label="",$fe=false)	{
		echo "<br />---start dump----- $label -----start dump---<br />";
		if($fe)	{
			if(is_array($var)||is_object($var)) {
				foreach($var as $key=>$value) {
					echo "$key=>$value<br />";
					if(is_array($var)||is_object($var))
					{
						ddump($var," уровень ");
					}
				}
			}	else {var_dump($var);
			}
		} else { var_dump($var);
		}
		echo "<br />---end dump------- $label -------end dump---<br />";
	}
	/**
	 * Исследователькая функция . Возвращает ассоциативный массив свойств и значений объекта
	 * @param $obj исследуемый объект
	 */
	public static function getProperties($obj=false){
		if($obj) {
			$class_methods = get_class_methods(get_class($obj));
			self::showArray($class_methods," Объект:  ".get_class($obj) );
		}
	}

	/**
	 *
	 * Преобразование строк вида %D0%BF%D1%80 в 'пр'
	 * @param unknown_type $text - входящий текст
	 * return  преобразованный текст
	 */
	public static function getTrasserText($text) {
		while (ereg('%([0-9A-F]{2})',$text)) {
			$val=ereg_replace('.*%([0-9A-F]{2}).*','\1',$text);
			$newval=chr(hexdec($val)); // получаем сивол с номером,
			$text=str_replace('%'.$val,$newval,$text);
		}
		return $text;
	}
	public static function getArrayOfNumbers($param) {
		$arr=array();
		if($param) {
			for ($i = 1; $i <=$param; $i++) {
				$arr[$i]=$i;
			}
		}
		return $arr;
	}
	public static function dsPath($path, $prefix=false) {
		$path=str_replace("/", DS, $path);
		if ($prefix) return str_replace(DS.DS,DS,$prefix.$path);
		return $path;
	}
	private static function stdSort($f1,$f2) {
		$fieldname=Util::$sortField;
		if($f1->$fieldname < $f2->$fieldname) return (self::$sortDir? -1: 1);
		elseif($f1->$fieldname > $f2->$fieldname) return (self::$sortDir? 1: -1);
		else return 0;
	}

	public static function sortStdClassArray(&$array,$fieldname,$asc=true) 	{
		self::$sortField=$fieldname;
		self::$sortDir=$asc;
		return uasort($array,array("Util","stdSort"));
	}
	public static function sendHeader($code=0){
		switch($code){
			case 301:
				header('HTTP/1.1 301 Moved Permanently');
				break;
			case 403:
				header('HTTP/1.1 403 Forbidden');
				break;
			case 404:
				header('HTTP/1.0 404 Not Found');
				break;
			case 500:
				header('HTTP/1.1 500 Internal Server Error');
				break;
			case 503:
				header("HTTP/1.1 503 Service Temporarily Unavailable");
				break;
			default:
				break;
		}
	}
}
?>