<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Widget extends Object {

	private $_models		= array();
	private $_data			= null;

	private static $_installedWidgets	= array();
	private static $_widgets			= array();

	public function getParamsMask(){ 
		$params=array();
		$params["Widget_ID"]["vtype"]="string"; $params["Widget_ID"]["vdefault"]="";
		return $params;
	}
	
	public static function initialize() {
		$db = Database::getInstance();
		if (defined("_ADMIN_MODE")) $db->setQuery("SELECT w_id, w_name FROM #__widgets ORDER BY w_name");
		else $db->setQuery("SELECT * FROM #__widgets WHERE w_side=1 ORDER BY w_name");
		self::$_installedWidgets = $db->loadObjectList("w_name");
	}

	public static function getInstance($widgetName) {
		if (array_key_exists($widgetName,self::$_widgets) == false) {
			if (array_key_exists($widgetName,self::$_installedWidgets)) { 

				// Local filesystem (PHP code)
				$widgetScriptPath = Portal::getInstance()->getTemplatePath().'widgets'.DS.$widgetName.DS.'default.php';
				Debugger::getInstance()->message("Widget load attempt - ".$widgetName." - &gt; ".$widgetScriptPath);
				if (!file_exists($widgetScriptPath)) {
					$widgetScriptPath = PATH_WIDGETS.$widgetName.DS.'default.php';
					Debugger::getInstance()->milestone("Widget load attempt - ".$widgetName." - &gt; ".$widgetScriptPath);
					if (!file_exists($widgetScriptPath) && DEFINED('PATH_FRONT_WIDGETS')) { // искали виджет в админке и не нашли
						$widgetScriptPath = PATH_FRONT_WIDGETS.$widgetName.DS.'default.php';
						Debugger::getInstance()->message("Widget load attempt - ".$widgetName." - &gt; ".$widgetScriptPath);
					}
					if (!file_exists($widgetScriptPath)) {
						Util::fatalError(Text::_("Widget not found")." => ".$widgetName);//$widgetName);
					}
				}
				require_once $widgetScriptPath;

				$widgetClass = $widgetName.'Widget';
				self::$_widgets[$widgetName] = new $widgetClass($widgetName,self::$_installedWidgets[$widgetName]);
			}	else Util::fatalError(Text::_('Widget is not installed').": $widgetName");
		}

		return self::$_widgets[$widgetName];
	}

	public static function isInstalled($widgetName) {
		return array_key_exists($widgetName,self::$_installedWidgets);
	}

	private function __construct($name,$data) {
		$this->initObj('widget'.$name);

		// Copy data from DB
		$this->_data = $data;

		// Try to load CSS
		$cssPath = "widgets/".$name.".css";
		Portal::getInstance()->addStyleSheet($cssPath);

		// Try to load JS
		$jsPath = "widgets/".$name.".js";
		Portal::getInstance()->addScript($jsPath);

		// Localization
		Text::parseWidget($name);

	}

	protected function getModel($moduleName,$modelName='') {
		$widgetName = $this->getName();

		if ($modelName == '') {
			$modelName = $widgetName;
		}

		$module = Module::getInstance($moduleName);

		$modelKey = $moduleName."_".$modelName;
		if (array_key_exists($modelKey,$this->_models) == false) {
			// Load model script
			$modelScriptPath = PATH_MODULES.DS.$moduleName.DS.'models'.DS.$modelName.'.php';
			if (!file_exists($modelScriptPath)) {
				$this->fatalError("Model not found => ".$moduleName.".".$modelName);
			}

			require_once $modelScriptPath;

			$modelClass = $moduleName.'Model'.$modelName;
			$this->_models[$modelKey] = new $modelClass($module);

			$this->message("Model loaded from widget => ".$moduleName.".".$this->getName().".".$modelName);
		}

		return $this->_models[$modelKey];
	}
	public function getHelper($helperName,$moduleName) {
		return Module::getHelper($helperName,$moduleName);
	}
	public function isActive() {
		return ($this->_data->w_active == '1');
	}
	public function checkSelf($haltOnFail=false) {
		return true;
	}
	public function render() {
		// Just for overriding
	}
	public function prepare() {
		// Just for overriding
	}
	public function curve() {
		$method=$this->get("method","render_default");
		$title = $this->get('title','');
//		$title_link = $this->get('title_link','');
		$show_title = $this->get('show_title',0);
		if (!$show_title)	$title="";
		$body=$this->render();
		if (!$body) return "";
		if (class_exists("widgetTMPL",false)&& method_exists("widgetTMPL", $method)) return widgetTMPL::$method($title,$body,$this); else return $body;
	}
	public static function getInstalled() {
		return self::$_installedWidgets;
	}
	public static function getInstalledForFront() {
		$db = Database::getInstance();
		$db->setQuery("SELECT * FROM #__widgets WHERE w_side=1 ORDER BY w_name");
		return $db->loadObjectList("w_name");
	}
	public function getParam($param_name) {
		return $this->get($param_name);
	}
	public static function cleanWidgetCache($widget_id, $widget_name){
		if (!$widget_id || !$widget_name) return;
		$files=Files::getFiles(PATH_CACHE."widgets".DS);
		$cache_mask="/".$widget_name."_".$widget_id."_(.*).php/";
		if ($files){
			foreach ($files as $file){
				if (preg_match($cache_mask, $file["filename"])) Files::delete(PATH_CACHE."widgets".DS.$file["filename"], true);
			}
		}
	}
}
?>