<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Database extends Object {
	//---------- Singleton implementation ------------
	private static $_instance = null;
	public static function createInstance($critFail=true) {
		if (self::$_instance == null) {
			self::$_instance = new self($critFail);
		}
	}
	public static function getInstance($critFail=true) {
		self::createInstance($critFail);
		return self::$_instance;
	}
	//------------------------------------------------
	private $_resource		= '';
	private $_connected		= false;
	private $_connect_timeout	= 20;
	private $_connect_timeout_default = 20;
	private $_dbHost		= '';
	private $_dbPort		= '';
	private $_dbName		= '';
	private $_dbUser		= '';
	private $_dbPrefix		= '';
	private $_sql			= '';
	private $_cursor		= null;
	private $_limit			= 0;
	private $_offset		= 0;
	private $_delimiter ="###qb_delimiter###";
//	private $_delimiter ="{#qb_delimiter#}";
	public function setDebugMode(){
		$this->_debugger=siteConfig::$debugMode;
	}
	private function __construct($critFail) {
		$this->_connect_timeout_default=ini_get("mysql.connect_timeout");
		@ini_set("mysql.connect_timeout",$this->_connect_timeout);
		$this->initObj();
		$this->reconnect(DatabaseConfig::$dbHost,DatabaseConfig::$dbPort,DatabaseConfig::$dbName,DatabaseConfig::$dbUser,DatabaseConfig::$dbPassword,DatabaseConfig::$dbPrefix,$critFail);
		$this->milestone("DB constructed");
	}
	private function replacePrefix($sql,$prefix="#__") {
		$sql = trim($sql);
		$escaped = false;
		$quoteChar = '';
		$n = strlen($sql);
		$startPos = 0;
		$literal = '';
		while($startPos < $n) {
			$ip = strpos($sql,$prefix,$startPos);
			if ($ip === false) break;
			$j = strpos($sql,"'",$startPos);
			$k = strpos($sql,'"',$startPos);
			if (($k !== false) && (($k < $j) || $j == false)) {
				$quoteChar = '"';
				$j = $k;
			}	else {
				$quoteChar = "'";
			}
			if ($j === false) $j = $n;
			$literal .= str_replace($prefix,$this->_dbPrefix,substr($sql,$startPos,$j - $startPos));
			$startPos = $j;
			$j = $startPos + 1;
			if ($j >= $n) break;
			while (true) {
				$k = strpos($sql,$quoteChar,$j);
				$escaped = false;
				if ($k === false) break;
				$l = $k - 1;
				while ($l >= 0 && $sql{$l} == '\\') {
					$l--;
					$escaped = !$escaped;
				}
				if ($escaped) {
					$j = $k + 1;
					continue;
				}
				break;
			}
			if ($k == false) { // error in the query - no end quote; ignore it
				break;
			}
			$literal .= substr($sql,$startPos,$k - $startPos + 1);
			$startPos = $k + 1;
		}
		if ($startPos < $n) $literal .= substr($sql,$startPos,$n - $startPos);
		return $literal;
	}

	function getNumRows( $cur=null ) {
		return mysql_num_rows( $cur ? $cur : $this->_cursor );
	}
	public function isConnected() {
		return $this->_connected;
	}
	public function getPrefix() {
		return $this->_dbPrefix;
	}
	public function getDBName() {
		return $this->_dbName;
	}
	public function getDelimiter() {
		return $this->_delimiter;
	}
	public function reconnect($dbhost,$dbport,$dbname,$dbuser,$dbpass,$dbprefix,$critFail=true) {
		$this->_connected = false;
		if ($this->_resource) mysql_close($this->_resource);
		$this->_dbHost = $dbhost;
		$this->_dbPort = $dbport;
		$this->_dbName = $dbname;
		$this->_dbUser = $dbuser;
		$this->_dbPrefix = $dbprefix;
		if (!($this->_resource = @mysql_connect($dbhost.":".$dbport,$dbuser,$dbpass,true))) {
			if ($critFail) Util::halt("DB connect failed");
			else {
				$this->_connected = false;
				return false;
			}
		}
		if (!mysql_select_db($dbname,$this->_resource)) {
			if ($critFail) Util::halt("DB selection failed");
			else {
				$this->_connected = false;
				return false;
			}
		}
		$this->milestone("DB selected `".$dbname."`");
		@mysql_query("SET NAMES 'UTF8'",$this->_resource);
		$this->_connected = true;
	}

	public function setQuery($sql, $offset = 0, $limit = 0, $prefix='#__') {
		$this->_sql = $this->replacePrefix($sql,$prefix);
		$this->_limit = intval($limit);
		$this->_offset = intval($offset);
	}
	public function getQuery($format=false) {
		if ($format) return '<pre>'.htmlspecialchars($this->_sql).'</pre>';
		else return $this->_sql;
	}
	public function query() {
		if ($this->_limit > 0 && $this->_offset == 0) {
			$this->_sql .= "\n LIMIT ".$this->_limit;
		}	else if ($this->_limit > 0 || $this->_offset > 0) {
			$this->_sql .= "\n LIMIT ".$this->_offset.", ".$this->_limit;
		}
		if ($this->_debugger) $_startTime = microtime(1);
		$this->_cursor = @mysql_query($this->_sql,$this->_resource);
		if ($this->_debugger) Debugger::getInstance()->sqlQuery((microtime(1) - $_startTime)." : ".$this->_sql);
		if (!$this->_cursor) {
			$this->error(Text::_("DB error")." => query ".$this->_sql." -:- ".mysql_error($this->_resource));
			$this->_errorNum .= mysql_errno( $this->_resource ) . " ";
			$this->_errorMsg .= mysql_error( $this->_resource )." SQL=".$this->_sql."<br />";
			return false;
		}
		return $this->_cursor;
	}
	public function loadResult() {
		if (!($cur = $this->query())) return null;
		if ($row = @mysql_fetch_row($cur)) {
			$ret = $row[0];
		} else $ret = null;
		@mysql_free_result($cur);
		return $ret;
	}
	public function loadAssocList($key='') {
		if (!($cur = $this->query())) return null;
		$array = array();
		while ($row = @mysql_fetch_assoc($cur)) {
			if ($key) $array[$row[$key]] = $row;
			else $array []= $row;
		}
		@mysql_free_result($cur);
		return $array;
	}
	/* Load an array of single field results into an array */
	function loadResultArray($numinarray = 0) {
		if (!($cur = $this->query())) return null;
		$array = array();
		while ($row = mysql_fetch_row( $cur )) {
			$array[] = $row[$numinarray];
		}
		mysql_free_result( $cur );
		return $array;
	}
	public function loadObject(&$object) {
		if ($object != null) {
			if (!($cur = $this->query())) return false;
			if ($array = mysql_fetch_assoc($cur)) {
				@mysql_free_result($cur);
				foreach ($array as $key=>$value) {
					$object->$key = $value;
				}
				return true;
			} else return false;
		} else {
			if ($cur = $this->query()) {
				if ($object = @mysql_fetch_object($cur)) {
					@mysql_free_result($cur);
					return true;
				} else {
					$object = null;
					return false;
				}
			} else return false;
		}
	}
	public function loadObjectList($key='') {
		if (!($cur = $this->query())) return null;
		$array = array();
		while ($row = @mysql_fetch_object($cur)) {
			if ($key) $array[$row->$key] = $row;
			else $array[] = $row;
		}
		@mysql_free_result($cur);
		return $array;
	}

	public function insertid() {
		return mysql_insert_id( $this->_resource );
	}
	public function getListFieldfromTable($table, $full=false) {
		if ($full) $this->setQuery("SHOW FULL COLUMNS FROM #__".$table);
		else $this->setQuery("DESCRIBE #__".$table);
		return $this->LoadObjectList('Field');
	}
	public function doubleBaks($text){
		return str_replace("$$",urlencode("$$"),$text);
	}
	public function query_batch( $abort_on_error=true, $p_transaction_safe = false,$delimiter='') {
		if (!$delimiter) $delimiter=$this->_delimiter;
		$this->_errorNum = 0;
		$this->_errorMsg = '';
		if ($p_transaction_safe) {
			$mysql_major=Session::getVar('mysql_major');
			if(is_null($mysql_major)){
				$si = mysql_get_server_info( $this->_resource );
				preg_match_all( "/(\d+)\.(\d+)\.(\d+)/i", $si, $m );
				$mysql_major=$m[1];
				Session::setVar('mysql_major',$mysql_major);
			}
			if ($mysql_major<5) Util::halt("Wrong SQL version");
			$this->_sql = 'START TRANSACTION'.$delimiter. $this->_sql .$delimiter.'COMMIT'.$delimiter;
		}
//		$query_split = preg_split ("/[".$delimiter."]+/", $this->_sql); // бред, ибо класс !!!
//		$query_split = explode ($delimiter, $this->_sql); в принципе работает
		$query_split = preg_split ("/(".$delimiter.")/", $this->_sql,-1,PREG_SPLIT_NO_EMPTY);
		$error = 0;
		foreach ($query_split as $command_line) {
			$command_line = trim( $command_line );
			if ($command_line != '') {
				if ($this->_debugger) $_startTime = microtime(1);
				$this->_cursor = mysql_query( $command_line, $this->_resource );
				if ($this->_debugger) Debugger::getInstance()->sqlQuery((microtime(1) - $_startTime)." : ".$command_line);
				if (!$this->_cursor) {
					$error = 1;
					$this->_errorNum .= mysql_errno( $this->_resource ) . " ";
					$this->_errorMsg .= mysql_error( $this->_resource )." SQL=".$command_line."<br />";
					if ($abort_on_error) return $this->_cursor;
				}
			}
		}
		return $error ? false : true;
	}

	public function getLastError() {
		return mysql_error($this->_resource);
	}
}
?>