<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class Module extends Object {

	//--------------- Factory implementation --------------------
	private static $_loadedModules		= array();
	private static $_loadMode					= 'main';

	public function getParamsMask(){ 
		$params['Page_size']["vtype"]="integer";		$params['Page_size']["vdefault"]="0";
		$params['Admin_page_size']["vtype"]="integer";	$params['Admin_page_size']["vdefault"]="0";
		return $params; 
	}

	public function getACLTemplate($is_admin=true)
	{
		return array();
	}
	
	public function getLinksArray(&$counter,&$arr) {
		$counter++;
		$module=$this->getName();
		$arr[$module][$counter]['link']=Router::_("index.php?module=".$module, true);
		$arr[$module][$counter]['name']=Text::_($module);
		$arr[$module][$counter]['fullname']=Text::_($module);
		return true;	
	}
	public function getSitemapHTML($links=true) { return array("html"=>"","links"=>array()); }
	
	public static function getInstance($name='') {
		if (!$name) $name = Registry::getInstance()->get('module');
		// Check and load
		if (!array_key_exists($name,self::$_loadedModules)) {
			// Check in database and get it
			$db=Database::getInstance();
			$db->setQuery("SELECT * FROM #__modules WHERE m_name='".$name."' AND m_deleted=0");
			$moduleData=$db->loadObjectList();
			if (count($moduleData)!=1) Util::fatalError(Text::_("Module not installed")." => ".$name,"404");
			// Module main script path
			$moduleScriptPath = PATH_MODULES.$name.DS.'module.php';
			if (!file_exists($moduleScriptPath)) {
				$moduleScriptPath = PATH_MODULES.$name.DS.$name.'.php';
				if (!file_exists($moduleScriptPath)) {
					if (defined("PATH_FRONT_MODULES")) {
						$moduleScriptPath = PATH_FRONT_MODULES.$name.DS.'module.php';
						if (!file_exists($moduleScriptPath)) {
							$moduleScriptPath = PATH_FRONT_MODULES.$name.DS.'module.php';
							if (!file_exists($moduleScriptPath)) Util::fatalError(Text::_("Module front not found")." => ".$moduleScriptPath.'--'.$name);
						}
					}	else Util::fatalError(Text::_("Module not found")." => ".$moduleScriptPath.'--'.$name);
				}
			}
			// Load language file
			if (!defined("_ADMIN_MODE")) Text::parseModule($name); 
			Debugger::getInstance()->milestone("Including (require_once) ".$moduleScriptPath );
			require_once $moduleScriptPath;
			Debugger::getInstance()->milestone("Included (require_once) ".$moduleScriptPath );
			$moduleClass = $name.'Module';
			self::$_loadedModules[$name] = new $moduleClass($name,self::$_loadMode,$moduleData[0]);
			self::$_loadMode = 'api';
		}
		return self::$_loadedModules[$name];
	}

	//-----------------------------------------------------------

	public static function isInstalled($moduleName) {
		$db=Database::getInstance();
		$db->setQuery("SELECT COUNT(m_id) FROM #__modules WHERE m_name='".$moduleName."'");
		if ($db->loadResult()!=1) return false;
		return true;
	}

	public static function getInstalledModules($forMap=false) {
		// Check in database
		$db=Database::getInstance();
		$sql="SELECT DISTINCT m_name FROM #__modules";
		if ($forMap) $sql.=" WHERE m_incl_map=1 AND m_deleted=0";
		$db->setQuery($sql);
		return $db->loadResultArray();
	}

	private static $_config		= null;
	private static $_helpers	= array();

	public static function getHelper($helperName,$moduleName="") {
		if(!$moduleName) $moduleName=Module::getInstance()->getName();
		if (array_key_exists($helperName,self::$_helpers) == false) {
			$helperScriptPath = PATH_MODULES.$moduleName.DS.'helpers'.DS.$helperName.'.php';
			if (!file_exists($helperScriptPath)) {
				Util::fatalError(Text::_("Helper not found")." => ".$moduleName.".".$helperName."(".$helperScriptPath.")");
			}

			require_once $helperScriptPath;

			$helperClass = $moduleName.'Helper'.ucfirst($helperName);
			self::$_helpers[$helperName] = new $helperClass();
		}

		return self::$_helpers[$helperName];
	}

	public static function initModuleClass($className,$moduleName,$forceFront=false,$fatal=true) {
		  if(defined("_ADMIN_MODE")&&$forceFront) $scriptPath = PATH_FRONT_MODULES.$moduleName.DS.'classes'.DS.$className.'.php';
		  else $scriptPath = PATH_MODULES.$moduleName.DS.'classes'.DS.$className.'.php';
			if (!file_exists($scriptPath)) {
				if($fatal) Util::fatalError(Text::_("Module class not found")." => ".$moduleName.".".$className."(".$scriptPath.")");
				else return false;
			}
   		require_once $scriptPath;
      return true; 
	}
	
	//-----------------------------------------------------------

	protected $_controller		= null;
	private $_controllerName	= '';
	private $_defaultViewName	= 'default';
	private $_models			= array();
	private $_showBreadcrumb	= false;
	private $_disabled			= '';

	public function __construct($name,$mode,$data) {
		Debugger::getInstance()->milestone("Module start loading => ".$name);
		$this->initObj($name);
		$this->validate();
		$_aclObject = ACLObject::getInstance($name.'Module');
		if (!$_aclObject->canAccess()) {
			if ($name==siteConfig::$defaultModule) $codeerror="503"; else $codeerror="404";
			$this->fatalError(Text::_('Permission is absent for')." ".$_aclObject->getDescription());
		}	else {
			$config = $data->m_config;
			$this->_showBreadcrumb = $data->m_show_breadcrumb;
			$this->registerEvents();
			if ($mode == 'main') {
				$this->prepare();
				$this->loadConfig($config);
				$this->initData();
				$this->getController();
			}	else {
				$this->loadConfig($config);
			}
			// Javascript for module
			if (!Portal::getInstance()->isDisabled() && !Portal::getInstance()->inPrintMode()) { Portal::getInstance()->addScript("modules/".$name.".js"); }
			$this->milestone("Module loaded => ".$name);
		}
	}

	private function initData() {
		$controller	= Request::get('controller','default');
		$task		= Request::get('task','show');
		$view		= Request::get('view',$this->_defaultViewName);
		$layout		= Request::get('layout','default');

		$this->set('controller',$controller,true);
		$this->set('task',$task,true);
		$this->set('view',$view,true);
		$this->set('layout',$layout,true);
	}

	private function loadConfig($config) {
		$params=Params::parse($config);
		if(is_array($params)) $this->_config = $params; else $this->_config = array();
	}
	protected function registerEvent($eventName) {
//		Event::register($this->getName().".".$eventName);
	}
	public function getController() {
		// Load controller script
		$controllerName = $this->get('controller');
		if ($controllerName != $this->_controllerName) {
			$path = PATH_MODULES.$this->getName().DS.'controllers'.DS.$controllerName.'.php';
			if (!file_exists($path)) {
				$this->fatalError(Text::_("Controller not found")." => ".$this->getName().".".$controllerName);
			}
			Debugger::getInstance()->milestone("Including (require_once) ".$path );
			require_once $path;
			Debugger::getInstance()->milestone("Included (require_once) ".$path );
			$this->_controllerName = $controllerName;
			$controllerClass = $this->getName().'Controller'.$controllerName;
			$this->_controller = new $controllerClass($controllerName,$this);
				
			return $this->_controller;
		}
	}

	public function getModel($modelName="") {
		$moduleName = $this->getName();
		$viewName = $this->get('view');
		if (!$modelName) $modelName = $viewName;
		$this->message("Model get attempt => ".$moduleName.".".$this->getName().".".$modelName);
		if (array_key_exists($modelName,$this->_models) == false) {
			$this->milestone("Model load attempt => MODULE=&quot;".$moduleName."&quot; MODEL=&quot;".$modelName."&quot; VIEW=&quot;".$viewName."&quot;");
			$path = PATH_MODULES.$moduleName.DS.'models'.DS.$modelName.'.php';
			if (!file_exists($path)) {
				// FIXME убрать поиск моделей и т.д на фронтенде
				if (defined("PATH_FRONT_MODULES"))	{
					$path = PATH_FRONT_MODULES.$moduleName.DS.'models'.DS.$modelName.'.php';
					if (!file_exists($path)) $this->fatalError(Text::_("Model front not found")." => ".$moduleName.'--'.$modelName);
				}	else $this->fatalError(Text::_("Model not found")." => ".$moduleName.".".$modelName);
			}
			Debugger::getInstance()->milestone("Including (require_once) ".$path );
			require_once $path;
			Debugger::getInstance()->milestone("Included (require_once) ".$path );
			$modelClass = $moduleName.'Model'.$modelName;
			$this->_models[$modelName] = new $modelClass($this);
			$this->milestone("Model loaded => ".$moduleName.".".$this->getName().".".$modelName);
		}
		return $this->_models[$modelName];
	}

	public function getConfig() {
		return $this->_config;
	}

	public function getConfigValue($key, $defaultValue='', $empty2def=false) {
		if (array_key_exists($key,$this->_config)) {
			if (!$this->_config[$key] && $empty2def) return $defaultValue;
			else return $this->_config[$key];
		}	else {
			return $defaultValue;
		}
	}

	public function setBreadCrumbVisibility($status=1) {
		$this->_showBreadcrumb=$status;
	}

	public function showBreadcrumb() {
		return $this->_showBreadcrumb;
	}
	//-------------------- Interface methods --------------------
	public function registerEvents() {
		// Just for overriding
	}

	public function prepare() {
		// Just for overriding
	}

	public function render() {
		if($this->_disabled) echo $this->_disabled;
		else $this->_controller->executeTask();
	}

	public function ajax() {
		if($this->_disabled) echo $this->_disabled;
		else $this->_controller->executeAjax();
	}

//	public function renderPanel($panelName) {
//		return $this->_controller->getView()->renderPanel($panelName);
//	}
	public function getBreadCrumbArray() {
		return $this->_controller->getView()->getBreadCrumbArray();
	}
	public function renderBreadCrumb($force=false) {
		if ($this->showBreadcrumb()||$force) return $this->_controller->getView()->renderBreadCrumb();
		else return "";
	}
	public function setDefaultView($viewName) {
		$this->_defaultViewName = $viewName;
	}

	public function getDefaultView() {
		return $this->_defaultViewName;
	}

	public function _disable($msg="Module disabled"){
		if (!$this->_disabled) $this->_disabled="<div id=\"disabled-message\">".$msg."</div>";
	}

	private function validate() {
		if (in_array($this->getName(),Portal::getInstance()->getDisabledModules())) {
			$this->_disabled = "<div id=\"disabled_message\">".
				Text::_('Module is disabled in this version')." (".Portal::getLicenseType().")</div>";
		}
	}
}

?>