<?php
/*
 * AltaDoc-CMS
 * Copyright (c) AltaDoc Group
 * Web: http://altadoc.ru
 * Commercial license http://altadoc.ru/article/litsenzionnoe_soglashenie.html
 * Last Changed Rev: 279
 * Build: 157
 * Build date: 2014.04.09 0:18:56
 * 
 */
defined('_ALTADOC_VALID') or die("Access denied");

class editorPluginbbcode extends Plugin {
	protected $_events=array("bbcode.editor","bbcode.parse");
	private $bbtags=array( "tag_p","tag_br","tag_b","tag_i","tag_s","tag_u","tag_url","tag_img","tag_size","tag_color","tag_ul","tag_ol","tag_code","tag_h");
	private $disabled_tags=array();
	public function getParamsMask(){
		foreach($this->bbtags as $tag){
			$params[$tag]["vtype"]="boolean";			$params[$tag]["vdefault"]=0;
		}
		return $params;
	}
	protected function onRaise($event, &$data) {
		foreach($this->bbtags as $tag){
			if (!$this->get($tag,0)) $this->disabled_tags[$tag]=true;
		}
		switch($event){
			case "bbcode.editor":
				$editor_id=$this->get("element_id");
				if ($editor_id){
					$dts=""; $dts_arr=array();
					foreach($this->disabled_tags as $dt=>$val) {
						if ($dt=="tag_h") $dts_arr[]="tag_h1:false,tag_h2:false,tag_h3:false,tag_h4:false,tag_h5:false,tag_h6:false";
						else $dts_arr[]=$dt.":false";
					}
					if (count($dts_arr)) $dts="{".implode(",",$dts_arr)."}";
					Portal::getInstance()->addScriptDeclaration("$(document).ready(function() { $('#".$editor_id."').bbcode_editor(".$dts."); });");
					Portal::getInstance()->addScript("/redistribution/bbcode/jquery.bbcode_editor.js");
					Portal::getInstance()->addStylesheet("/redistribution/bbcode/jquery.bbcode_editor.css");
				}
				break;
			case "bbcode.parse":
				$this->parseBBCode($data);
				break;
		}
	}

	private function parseBBCode(&$text) {
		$key="tag_br";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[br\]#is";
			$replacements []= "<br />";
		}
		$key="tag_p";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[p\](.+?)\[\/p\]#is";
			$replacements []= "<p>\\1</p>";
		}
		$key="tag_b";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[b\](.+?)\[\/b\]#is";
			$replacements []= "<strong>\\1</strong>";
		}
		$key="tag_i";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[i\](.+?)\[\/i\]#is";
			$replacements []= "<span style='font-style:italic'>\\1</span>";
		}
		$key="tag_s";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[s\](.+?)\[\/s\]#is";
			$replacements []= "<span style='text-decoration:line-through'>\\1</span>";
		}
		$key="tag_u";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[u\](.+?)\[\/u\]#is";
			$replacements []= "<span style='text-decoration:underline'>\\1</span>";
		}
		$key="tag_url";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[url=http\:\/\/(.+?)\](.+?)\[\/url\]#is";
			$replacements []= "<a target=\"_blank\" href='http://\\1'>\\2</a>";

			$patterns []= "#\[url=http\:\/\/(.+?)\]\[\/url\]#is";
			$replacements []= "<a target=\"_blank\" href='http://\\1'>\\1</a>";

			$patterns []= "#\[url=(.+?)\](.+?)\[\/url\]#is";
			$replacements []= "<a target=\"_blank\" href='\\1'>\\2</a>";

			$patterns []= "#\[url=(.+?)\]\[\/url\]#is";
			$replacements []= "<a target=\"_blank\" href='\\1'>\\1</a>";
		}
		$key="tag_img";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[img\](.+?)\[\/img\]#is";
			$replacements []= "<img alt=\"\" src='\\1' />";
		}
		$key="tag_size";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[size=(.+?)\](.+?)\[\/size\]#is";
			$replacements []= "<span style='font-size:\\1pt'>\\2</span>";
		}
		$key="tag_color";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[color=(.+?)\](.+?)\[\/color\]#is";
			$replacements []= "<span style='color:\\1'>\\2</span>";
		}
		$key="tag_ul";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[list\](.+?)\[\/list\]#is";
			$replacements []= "<ul>\\1</ul>";
		}
		$key="tag_ol";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[list=(1|a|I)\](.+?)\[\/list\]#is";
			$replacements []= "<ol type='\\1'>\\2</ol>";
		}
		if ((!array_key_exists("tag_ol",$this->disabled_tags))|| (!array_key_exists("tag_ul",$this->disabled_tags)))	{
			$patterns []= "#\[\*\](.*)#";
			$replacements []= "<li>\\1</li>";
		}
		$key="tag_code";
		// @TODO CODE tag
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[code\](.+?)\[\/code\]#is";
			$replacements []= "<pre class='program_code'>\\1</pre>";
		}
		$key="tag_h";
		if (!array_key_exists($key,$this->disabled_tags)){
			$patterns []= "#\[h(1|2|3|4|5|6)\](.+?)\[/h\\1\]#is";
			$replacements []= "<h\\1>\\2</h\\1>";
		}
		if (count($patterns)) $text = preg_replace($patterns,$replacements,$text);
		$text = str_replace("\n","<br />",$text);

		return $text;
	}
}
?>