<?php 
/**
 * Vkontakte API authentication using OAuth 2.
 *
 * @category Tools
 * @package VK
 * @author Radius17 <info@radius17.ru>
 * @copyright 2009-2012 Radius17 <info@radius17.ru>
 * @license http://www.opensource.org/licenses/mit-license.php MIT
 * @link http://radius17.ru/
 */
 
class VK_OAuth {
	private $vk_key=false;
	private $vk_secret=false;
	private $auth_uri="http://api.vk.com/oauth/authorize?";
	private $profile_uri="https://api.vkontakte.ru/method/getProfiles?";
	private $token_uri="https://oauth.vkontakte.ru/access_token?";

	public $connecttimeout = 30; 
  public $timeout = 30; 
  public $ssl_verifypeer = false;
  public $format = 'json';
  public $decode_json = true;
  public $useragent = 'Vkontakte OAuth';
  
  public $http_code=0;
  public $http_info=array();
  public $http_url="";
  			
	private $oauth_code="";
	private $oauth_token="";
	private $user_id="";
	public $error_code="";
	public $error_desc="";
	
	public function __construct($vk_key, $vk_secret){
		$this->vk_key=$vk_key;
		$this->vk_secret=$vk_secret;
	}
	public function getLoginURI($callback_uri){
		$params=array("redirect_uri"=>$callback_uri, "response_type"=>"code", "client_id"=>$this->vk_key);
		return $this->auth_uri.http_build_query($params);
	}
	public function validateAuthResponse(){
	  $this->error_desc=$this->getSafeRequest("error_description");
		$this->error_code=$this->getSafeRequest("error");
 		$this->oauth_code=$this->getSafeRequest("code","");
 		if ($this->error_code) return false;
 		if (!$this->oauth_code) return false;
 		if (!$this->getAccessToken()) return false;
		return true;
	}
	public function getProfile(){
		if ($this->user_id && $this->oauth_token) {
			$url=$this->profile_uri."uid=".$this->user_id."&access_token=".$this->oauth_token;
			$response=$this->get($url);
			if (isset($response["response"][0]["first_name"])&&isset($response["response"][0]["last_name"])&&isset($response["response"][0]["uid"])) {
				$profile["id"]=$response["response"][0]["uid"];
				$profile["name"]=$response["response"][0]["first_name"]." ".$response["response"][0]["last_name"]; 
				return $profile;
			} else return false;
		} else return false;
	}
	private function getAccessToken(){
		$url=$this->token_uri."client_id=".$this->vk_key."&client_secret=".$this->vk_secret."&code=".$this->oauth_code;
		$response=$this->get($url);
		if (isset($response["access_token"])&&isset($response["user_id"])&&$response["user_id"]) {
			$this->oauth_token=$response["access_token"];
			$this->user_id=$response["user_id"];
			return true;
		}
		else return false;
	}
	private function getSafeRequest($key="",$def=""){
		if (!$key ||!isset($_REQUEST[$key])) return $def;
		else return htmlspecialchars($_REQUEST[$key],ENT_COMPAT,'UTF-8');
	}
	private function get($url, $parameters = array()) {
		$response = $this->http($url, 'GET', $parameters);
		if ($this->format === 'json' && $this->decode_json) {
			return json_decode($response,true);
		}
		return $response;
	}
	private function http($url, $method, $postfields = NULL) {
		$this->http_info = array();
		$ci = curl_init();
		/* Curl settings */
		curl_setopt($ci, CURLOPT_USERAGENT, $this->useragent);
		curl_setopt($ci, CURLOPT_CONNECTTIMEOUT, $this->connecttimeout);
		curl_setopt($ci, CURLOPT_TIMEOUT, $this->timeout);
		curl_setopt($ci, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ci, CURLOPT_HTTPHEADER, array('Expect:'));
		curl_setopt($ci, CURLOPT_SSL_VERIFYPEER, $this->ssl_verifypeer);
		curl_setopt($ci, CURLOPT_HEADER, false);
	
		switch ($method) {
			case 'POST':
				curl_setopt($ci, CURLOPT_POST, true);
				if (!empty($postfields)) {
					curl_setopt($ci, CURLOPT_POSTFIELDS, $postfields);
				}
				break;
		}
	
		curl_setopt($ci, CURLOPT_URL, $url);
		$response = curl_exec($ci);
		$this->http_code = curl_getinfo($ci, CURLINFO_HTTP_CODE);
		curl_close ($ci);
		return $response;
	}
}
 ?>